#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.decorator import reify
from pyramid.view import view_config
from zope.interface import Interface

from pyams_app_msc.interfaces import MANAGE_THEATER_PERMISSION, VIEW_THEATER_PERMISSION
from pyams_app_msc.shared.theater import ICinemaPriceContainerTarget
from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_app_msc.shared.theater.interfaces.price import ICinemaPrice, ICinemaPriceContainer
from pyams_app_msc.shared.theater.zmi.interfaces import ICinemaPricesTable
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.interfaces.view import IModalEditForm
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm, SimpleAddFormRenderer, SimpleEditFormRenderer
from pyams_zmi.helper.container import delete_container_element, switch_element_attribute
from pyams_zmi.interfaces import IAdminLayer, IObjectHint, IObjectLabel
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IPropertiesMenu, IToolbarViewletManager
from pyams_zmi.table import AttributeSwitcherColumn, NameColumn, ReorderColumn, SortableTable, TableAdminView, \
    TableElementEditor, TrashColumn
from pyams_zmi.utils import get_object_label
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewlet_config(name='cinema-prices.menu',
                context=ICinemaPriceContainerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=525,
                permission=VIEW_THEATER_PERMISSION)
class CinemaPricesMenu(NavigationMenuItem):
    """Cinema prices menu"""

    label = _("Cinema prices")
    href = '#cinema-prices.html'


@factory_config(ICinemaPricesTable)
class CinemaPricesTable(SortableTable):
    """Cinema prices table"""

    container_class = ICinemaPriceContainer

    display_if_empty = True


@adapter_config(required=(ICinemaPriceContainerTarget, IAdminLayer, ICinemaPricesTable),
                provides=IValues)
class CinemaPricesTableValues(ContextRequestViewAdapter):
    """Cinema prices table values adapter"""

    @property
    def values(self):
        """Cinema prices values getter"""
        yield from ICinemaPriceContainer(self.context).values()


@adapter_config(name='reorder',
                required=(ICinemaPriceContainerTarget, IAdminLayer, ICinemaPricesTable),
                provides=IColumn)
class CinemaPricesTableReorderColumn(ReorderColumn):
    """Cinema price table reorder column"""


@view_config(name='reorder.json',
             context=ICinemaPriceContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=MANAGE_THEATER_PERMISSION)
def reorder_cinema_prices_table(request):
    """Reorder cinema price table"""
    order = request.params.get('order').split(';')
    request.context.updateOrder(order)
    return {
        'status': 'success',
        'closeForm': False
    }


@adapter_config(name='active',
                required=(ICinemaPriceContainerTarget, IAdminLayer, ICinemaPricesTable),
                provides=IColumn)
class CinemaPricesActiveColumn(AttributeSwitcherColumn):
    """Cinema prices table active column"""

    attribute_name = 'active'
    attribute_switcher = 'switch-active-price.json'

    icon_off_class = 'far fa-eye-slash text-danger'

    permission = MANAGE_THEATER_PERMISSION
    weight = 1

    def get_icon_hint(self, item):
        """Icon hint getter"""
        if self.has_permission(item):
            hint = _("Click icon to switch price activity")
        elif item.active:
            hint = _("This price is active")
        else:
            hint = _("This price is not active")
        return self.request.localizer.translate(hint)


@view_config(name='switch-active-price.json',
             context=ICinemaPriceContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def switch_active_price(request):
    """Switch cinema price activity flag"""
    return switch_element_attribute(request, container_factory=ICinemaPriceContainer)


@adapter_config(name='label',
                required=(ICinemaPriceContainerTarget, IAdminLayer, ICinemaPricesTable),
                provides=IColumn)
class CinemaPriceLabelColumn(NameColumn):
    """Cinema prices table label column"""

    attr_name = 'name'
    i18n_header = _("Price name")


@adapter_config(name='trash',
                required=(ICinemaPriceContainerTarget, IAdminLayer, ICinemaPricesTable),
                provides=IColumn)
class CinemaPriceTrashColumn(TrashColumn):
    """Cinema prices table trash column"""

    object_data = {
        'ams-modules': 'container',
        'ams-delete-target': 'delete-cinema-price.json'
    }


@view_config(name='delete-cinema-price.json',
             context=ICinemaPriceContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=MANAGE_THEATER_PERMISSION)
def delete_cinema_price(request):
    """Delete cinema price"""
    return delete_container_element(request, container_factory=ICinemaPriceContainer)


@pagelet_config(name='cinema-prices.html',
                context=ICinemaPriceContainerTarget, layer=IPyAMSLayer,
                permission=VIEW_THEATER_PERMISSION)
class CinemaPricesView(TableAdminView):
    """Cinema prices view"""

    title = _("Cinema prices")

    table_class = ICinemaPricesTable
    table_label = _("Cinema prices list")


#
# Cinema prices components
#

@viewlet_config(name='add-cinema-price.action',
                context=ICinemaPriceContainerTarget, layer=IAdminLayer, view=ICinemaPricesTable,
                manager=IToolbarViewletManager, weight=20,
                permission=MANAGE_THEATER_PERMISSION)
class CinemaPriceAddAction(ContextAddAction):
    """Cinema price add action"""

    label = _("Add price")
    href = 'add-cinema-price.html'


@ajax_form_config(name='add-cinema-price.html',
                  context=ICinemaPriceContainerTarget, layer=IPyAMSLayer,
                  permission=MANAGE_THEATER_PERMISSION)
class CinemaPriceAddForm(AdminModalAddForm):
    """Cinema price add form"""

    subtitle = _("New cinema price")
    legend = _("New cinema price properties")

    content_factory = ICinemaPrice
    fields = Fields(ICinemaPrice).omit('__parent__', '__name__', 'active')

    def add(self, obj):
        ICinemaPriceContainer(self.context).append(obj)


@adapter_config(required=(ICinemaPriceContainerTarget, IAdminLayer, CinemaPriceAddForm),
                provides=IFormTitle)
def cinema_price_add_form_title(context, request, form):
    """Cinema price add form title"""
    return '<span class="tiny">{}</span>'.format(
        get_object_label(context, request, form))


@adapter_config(required=(ICinemaPriceContainerTarget, IAdminLayer, CinemaPriceAddForm),
                provides=IAJAXFormRenderer)
class CinemaPriceAddFormRenderer(SimpleAddFormRenderer):
    """Cinema price add form renderer"""

    table_factory = ICinemaPricesTable


@adapter_config(required=(ICinemaPrice, IAdminLayer, Interface),
                provides=IObjectLabel)
def cinema_price_label(context, request, view):
    """Cinema price label"""
    return context.name


@adapter_config(required=(ICinemaPrice, IAdminLayer, Interface),
                provides=IObjectHint)
def cinema_price_hint(context, request, view):
    """Cinema price hint"""
    return request.localizer.translate(_("Cinema price"))


@adapter_config(required=(ICinemaPrice, IAdminLayer, ICinemaPricesTable),
                provides=ITableElementEditor)
class CinemaPriceEditor(TableElementEditor):
    """Cinema price editor"""


@ajax_form_config(name='properties.html',
                  context=ICinemaPrice, layer=IPyAMSLayer,
                  permission=VIEW_THEATER_PERMISSION)
class CinemaPricePropertiesEditForm(AdminModalEditForm):
    """Cinema price properties edit form"""

    legend = _("Cinema price properties")

    fields = Fields(ICinemaPrice).omit('__parent__', '__name__', 'active')


@adapter_config(required=(ICinemaPrice, IAdminLayer, IModalEditForm),
                provides=IFormTitle)
def cinema_price_edit_form_title(context, request, form):
    translate = request.localizer.translate
    theater = get_parent(context, IMovieTheater)
    return '<span class="tiny">{}</span><br />{}'.format(
        get_object_label(theater, request, form),
        translate(_("Cinema price: {}")).format(get_object_label(context, request, form)))


@adapter_config(required=(ICinemaPrice, IAdminLayer, CinemaPricePropertiesEditForm),
                provides=IAJAXFormRenderer)
class CinemaPricePropertiesEditFormRenderer(SimpleEditFormRenderer):
    """Cinema price properties edit form renderer"""

    parent_interface = ICinemaPriceContainerTarget
    table_factory = ICinemaPricesTable



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface

from pyams_app_msc.feature.tmdb import ITMDBConfiguration
from pyams_content.interfaces import MANAGE_SITE_ROOT_PERMISSION
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormContent, IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, FormGroupChecker
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuDivider, NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewlet_config(name='tmdb-configuration.divider',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=899,
                permission=MANAGE_SITE_ROOT_PERMISSION)
class TMDBConfigurationMenuDivider(NavigationMenuDivider):
    """TMDB configuration menu divider"""


@viewlet_config(name='tmdb-configuration.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=900,
                permission=MANAGE_SITE_ROOT_PERMISSION)
class TMDBConfigurationMenu(NavigationMenuItem):
    """TMDB configuration menu"""

    label = _("TMDB configuration")
    href = '#tmdb-configuration.html'


@ajax_form_config(name='tmdb-configuration.html',
                  context=ISiteRoot, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_ROOT_PERMISSION)
class TMDBConfigurationEditForm(AdminEditForm):
    """TMDB configuration edit form"""

    title = _("TMDB service configuration")
    legend = _("TMDB service properties")

    fields = Fields(Interface)


@adapter_config(required=(ISiteRoot, IPyAMSLayer, TMDBConfigurationEditForm),
                provides=IFormContent)
def site_root_tmdb_configuration_edit_form_content(context, request, form):
    """Site root TMDB configuration edit form content getter"""
    return ITMDBConfiguration(context)


@adapter_config(name='tmdb-configuration',
                required=(ISiteRoot, IAdminLayer, TMDBConfigurationEditForm),
                provides=IGroup)
class TMDBConfigurationGroup(FormGroupChecker):
    """TMDB configuration group"""

    fields = Fields(ITMDBConfiguration).omit('proxy_info',
                                             'download_main_poster', 'main_poster_size',
                                             'download_pictures', 'pictures_size',
                                             'download_posters', 'posters_size',
                                             'download_videos', 'restrict_videos_language') + \
        Fields(ITMDBConfiguration).select('proxy_info')


@adapter_config(name='tmdb-main-poster-configuration',
                required=(ISiteRoot, IAdminLayer, TMDBConfigurationGroup),
                provides=IGroup)
class TMDBMainPosterConfiguration(FormGroupChecker):
    """TMDB main poster configuration group"""

    weight = 10

    fields = Fields(ITMDBConfiguration).select('download_main_poster', 'main_poster_size')


@adapter_config(name='tmdb-pictures-configuration',
                required=(ISiteRoot, IAdminLayer, TMDBConfigurationGroup),
                provides=IGroup)
class TMDBPicturesConfiguration(FormGroupChecker):
    """TMDB pictures configuration group"""

    weight = 20

    fields = Fields(ITMDBConfiguration).select('download_pictures', 'pictures_size')


@adapter_config(name='tmdb-posters-configuration',
                required=(ISiteRoot, IAdminLayer, TMDBConfigurationGroup),
                provides=IGroup)
class TMDBPostersConfiguration(FormGroupChecker):
    """TMDB pictures configuration group"""

    weight = 30

    fields = Fields(ITMDBConfiguration).select('download_posters', 'posters_size')


@adapter_config(name='tmdb-videos-configuration',
                required=(ISiteRoot, IAdminLayer, TMDBConfigurationGroup),
                provides=IGroup)
class TMDBVideosConfiguration(FormGroupChecker):
    """TMDB videos configuration group"""

    weight = 40

    fields = Fields(ITMDBConfiguration).select('download_videos', 'restrict_videos_language')



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_app_msc.shared.theater import IMovieTheater
from pyams_content.feature.seo import ISEOContentInfo, SEOContentInfo
from pyams_utils.adapter import adapter_config

__docformat__ = 'restructuredtext'


class MovieTheaterSEOContentInfo(SEOContentInfo):
    """Movie theater SEO content info"""

    include_sitemap = True


@adapter_config(required=IMovieTheater,
                provides=ISEOContentInfo)
def movie_theater_seo_content_info(context):
    return MovieTheaterSEOContentInfo()



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface, implementer
from zope.schema import Bool
from zope.schema.interfaces import IBool

from pyams_app_msc.shared.catalog import IWfCatalogEntry
from pyams_form.datamanager import AttributeField
from pyams_utils.adapter import adapter_config
from pyams_utils.interfaces.form import IDataManager

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class ITMDBLookupField(IBool):
    """TMDB lookup field interface"""


@implementer(ITMDBLookupField)
class TMDBLookupField(Bool):
    """TMDB lookup field"""


@adapter_config(required=(IWfCatalogEntry, ITMDBLookupField),
                provides=IDataManager)
class TMDBSearchDataManager(AttributeField):
    """TMDB search field data manager"""

    def can_access(self):
        return True

    def can_write(self):
        return True

    def get(self):
        return False

    def set(self, value):
        pass


class ITMDBSearchInterface(Interface):
    """TMDB search interface"""

    tmdb_lookup = TMDBLookupField(title=_("TMDB lookup"),
                                  description=_("If 'yes', a lookup is done in TMDB database to find "
                                                "movie properties"),
                                  required=True,
                                  default=True)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_app_msc.shared.theater.interfaces import IMovieTheater, IMovieTheaterCalendarPortalPage, \
    IMovieTheaterCatalogPortalPage, IMovieTheaterMoviesPortalPage
from pyams_portal.interfaces import IPortalPage, IPortalPortletsConfiguration, PORTAL_PAGE_KEY
from pyams_portal.page import PortalPage, portal_context_portlets_configuration
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.zodb import volatile_property

__docformat__ = 'restructuredtext'


#
# Movie theater calendar pages
#

class MovieTheaterCalendarPageMixin:
    """Movie theater calendar page mixin class"""

    @volatile_property
    def can_inherit(self):
        return False


@factory_config(IMovieTheaterCalendarPortalPage)
class MovieTheaterCalendarPortalPage(MovieTheaterCalendarPageMixin, PortalPage):
    """Movie theater calendar portal page"""


@adapter_config(name='calendar',
                required=IMovieTheater,
                provides=IPortalPage)
def movie_theater_calendar_page(context, page_name='calendar'):
    """Movie theater calendar page factory"""

    def set_page_name(page):
        """Set page name after creation"""
        page.name = page_name

    key = f'{PORTAL_PAGE_KEY}::{page_name}' if page_name else PORTAL_PAGE_KEY
    return get_annotation_adapter(context, key, IMovieTheaterCalendarPortalPage,
                                  name=f'++page++{page_name}',
                                  callback=set_page_name)


@adapter_config(name='calendar',
                required=IMovieTheater,
                provides=IPortalPortletsConfiguration)
def movie_theater_calendar_portlets_configuration(context):
    """Movie theater calendar portlets configuration"""
    return portal_context_portlets_configuration(context, page_name='calendar')


#
# Movie theater movies list pages
#

class MovieTheaterMoviesPageMixin:
    """Movie theater movies list page mixin class"""

    @volatile_property
    def can_inherit(self):
        return False


@factory_config(IMovieTheaterMoviesPortalPage)
class MovieTheaterMoviesPortalPage(MovieTheaterMoviesPageMixin, PortalPage):
    """Movie theater movies list portal page"""


@adapter_config(name='movies',
                required=IMovieTheater,
                provides=IPortalPage)
def movie_theater_movies_page(context, page_name='movies'):
    """Movie theater movies list page factory"""

    def set_page_name(page):
        """Set page name after creation"""
        page.name = page_name

    key = f'{PORTAL_PAGE_KEY}::{page_name}' if page_name else PORTAL_PAGE_KEY
    return get_annotation_adapter(context, key, IMovieTheaterMoviesPortalPage,
                                  name=f'++page++{page_name}',
                                  callback=set_page_name)


@adapter_config(name='movies',
                required=IMovieTheater,
                provides=IPortalPortletsConfiguration)
def movie_theater_movies_portlets_configuration(context):
    """Movie theater movies list portlets configuration"""
    return portal_context_portlets_configuration(context, page_name='movies')


#
# Movie theater catalog entries pages
#

class MovieTheaterCatalogPageMixin:
    """Movie theater catalog page mixin class"""

    @volatile_property
    def can_inherit(self):
        return False


@factory_config(IMovieTheaterCatalogPortalPage)
class MovieTheaterCatalogPortalPage(MovieTheaterCatalogPageMixin, PortalPage):
    """Movie theater catalog portal page"""


@adapter_config(name='catalog',
                required=IMovieTheater,
                provides=IPortalPage)
def movie_theater_catalog_page(context, page_name='catalog'):
    """Movie theater catalog page factory"""

    def set_page_name(page):
        """Set page name after creation"""
        page.name = page_name

    key = f'{PORTAL_PAGE_KEY}::{page_name}' if page_name else PORTAL_PAGE_KEY
    return get_annotation_adapter(context, key, IMovieTheaterCatalogPortalPage,
                                  name=f'++page++{page_name}',
                                  callback=set_page_name)


@adapter_config(name='catalog',
                required=IMovieTheater,
                provides=IPortalPortletsConfiguration)
def movie_theater_catalog_portlets_configuration(context):
    """Movie theater catalog portlets configuration"""
    return portal_context_portlets_configuration(context, page_name='catalog')



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface
from zope.schema import TextLine

from pyams_utils.schema import MailAddressField

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class IContact(Interface):
    """Contact interface"""

    name = TextLine(title=_("Full name"),
                    description=_("Contact full name"),
                    required=False)

    email_address = MailAddressField(title=_("Email address"),
                                     description=_("Contact email address"),
                                     required=False)

    phone_number = TextLine(title=_("Phone number"),
                            description=_("Contact phone number"),
                            required=False)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""
from collections import OrderedDict
from enum import Enum

from zope.interface import Attribute
from zope.schema import Choice
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_i18n.schema import I18nTextLineField
from pyams_portal.interfaces import IPortlet, IPortletSettings

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class THEATERS_ORDER(Enum):
    """Theaters carousel order"""
    ALPHA = 'alpha'
    RANDOM = 'random'


THEATERS_ORDER_LABEL = OrderedDict((
    (THEATERS_ORDER.ALPHA, _("Alphabetical order")),
    (THEATERS_ORDER.RANDOM, _("Random order"))
))


THEATERS_ORDER_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v.value, title=t)
    for v, t in THEATERS_ORDER_LABEL.items()
])


class IMovieTheaterCarouselPortletSettings(IPortletSettings):
    """Movie theaters carousel portlet settings interface"""

    title = I18nTextLineField(title=_("Title"),
                              description=_("Portlet main title"),
                              required=False)

    theaters = Attribute("Movie theaters iterator")

    theaters_order = Choice(title=_("Theaters display order"),
                            description=_("Choose order in which theaters are displayed"),
                            vocabulary=THEATERS_ORDER_VOCABULARY,
                            required=True,
                            default=THEATERS_ORDER.ALPHA.value)


class IAudiencesPortletSettings(IPortletSettings):
    """Audiences portlet settings interface"""

    def get_parent(self, context):
        """Get movie theater parent from given context"""

    def get_audiences(self, context):
        """Movie theater audiences iterator getter"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.view import view_config
from zope.interface import alsoProvides

from pyams_app_msc.feature.planning.interfaces import IMovieTheaterSession, IPlanning, ISession
from pyams_app_msc.feature.planning.zmi.session import SessionAddForm
from pyams_app_msc.interfaces import MANAGE_CATALOG_PERMISSION
from pyams_app_msc.shared.catalog.interfaces import CATALOG_ENTRY_CONTENT_TYPE
from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_app_msc.zmi import msc
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import USE_INTERNAL_API_PERMISSION
from pyams_sequence.api.rest import find_references
from pyams_sequence.reference import get_reference_target
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.interfaces.form import NO_VALUE_STRING
from pyams_utils.url import absolute_url

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


IMovieTheaterSession['activity'].content_type = CATALOG_ENTRY_CONTENT_TYPE


@view_config(name='find-references.json',
             context=IMovieTheater, request_type=IPyAMSLayer,
             permission=USE_INTERNAL_API_PERMISSION,
             renderer='json', xhr=True)
def find_theater_references(request):
    """Returns list of references matching given query inside theater context"""
    return find_references(request, parent=request.context, validate=False)


@ajax_form_config(name='add-session.html',
                  context=IMovieTheater, layer=IPyAMSLayer,
                  permission=MANAGE_CATALOG_PERMISSION)
class MovieTheaterSessionAddForm(SessionAddForm):
    """Movie theater session add form"""

    @property
    def fields(self):
        fields = Fields(ISession).omit('duration')
        if not self.request.POST:
            fields = Fields(IMovieTheaterSession).select('activity') + fields
        return fields

    content_factory = ISession

    planning_target = None

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        activity = self.widgets.get('activity')
        if activity is not None:
            activity.prompt = True
            activity.prompt_message = self.request.localizer.translate(_("Out of catalog activity"))
            activity.ajax_url = absolute_url(self.context, self.request, 'find-references.json')
            activity.object_data = {
                'ams-modules': {
                    'msc': {
                        'src': get_resource_path(msc)
                    }
                },
                'ams-change-handler': 'MyAMS.msc.session.changeActivity'
            }
            alsoProvides(activity, IObjectData)

    def create(self, data):
        self.planning_target = IPlanning(self.context, None)
        activity = self.request.params.get(f'{self.prefix}{self.widgets.prefix}activity')
        if activity != NO_VALUE_STRING:
            target = get_reference_target(activity)
            if target is not None:
                self.planning_target = IPlanning(target, None)
        return super().create(data)

    def add(self, obj):
        self.planning_target.add_session(obj)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.httpexceptions import HTTPForbidden, HTTPInternalServerError
from pyramid.view import view_config
from zope.copy import copy
from zope.interface import Interface

from pyams_app_msc.component.gallery.zmi.interfaces import IGalleryIllustrationActionsMenu
from pyams_app_msc.zmi import msc
from pyams_content.component.gallery import IGalleryContainer, IGalleryFile
from pyams_content.component.gallery.zmi import GalleryMediasViewlet, GalleryView
from pyams_content.component.illustration import IIllustration, ILinkIllustration
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.shared.common import IWfSharedContent
from pyams_i18n.interfaces import INegotiator
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.permission import get_edit_permission
from pyams_skin.interfaces.viewlet import IContextActionsViewletManager
from pyams_skin.viewlet.actions import ContextAction, ContextActionsMenu
from pyams_skin.viewlet.menu import MenuItem
from pyams_template.template import override_template
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewletmanager_config(name='set-content-illustration.menu',
                       context=IGalleryFile, layer=IAdminLayer, view=Interface,
                       manager=IContextActionsViewletManager, weight=100,
                       provides=IGalleryIllustrationActionsMenu,
                       permission=MANAGE_CONTENT_PERMISSION)
class GalleryFileIllustrationsGetter(ContextActionsMenu):
    """Gallery file illustrations getter"""

    def __new__(cls, context, request, view, manager):
        gallery = get_parent(context, IGalleryContainer)
        if gallery is not None:
            edit_permission = get_edit_permission(request, context=gallery, view=view)
            if not request.has_permission(edit_permission, context=context):
                return None
        return ContextActionsMenu.__new__(cls)

    # hint = _("Set content illustration")
    css_class = 'btn-sm px-1'
    icon_class = 'fas fa-file-image'

    def update(self):
        super().update()
        msc.need()


@viewlet_config(name='content-illustration.action',
                context=IGalleryFile, layer=IAdminLayer, view=Interface,
                manager=IGalleryIllustrationActionsMenu, weight=10,
                permission=MANAGE_CONTENT_PERMISSION)
class ContentIllustrationSetter(MenuItem):
    """Content illustration setter"""

    label = _("Set content illustration")

    def get_href(self):
        """Icon URL getter"""
        return None

    click_handler = 'MyAMS.msc.catalog.setContentIllustrationFromGallery'


@viewlet_config(name='link-illustration.action',
                context=IGalleryFile, layer=IAdminLayer, view=Interface,
                manager=IGalleryIllustrationActionsMenu, weight=20,
                permission=MANAGE_CONTENT_PERMISSION)
class LinkIllustrationSetter(MenuItem):
    """Link illustration setter"""

    label = _("Set navigation illustration")

    def get_href(self):
        """Icon URL getter"""
        return None

    click_handler = 'MyAMS.msc.catalog.setLinkIllustrationFromGallery'


def set_illustration(request, illustration_interface):
    """Set content illustration"""
    translate = request.localizer.translate
    name = request.params.get('object_name')
    if not name:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'message': translate(_("No provided object_name argument!"))
            }
        }
    container = request.context
    if name not in container:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'message': translate(_("Given element name doesn't exist!"))
            }
        }
    content = get_parent(container, IWfSharedContent)
    if content is None:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'message': translate(_("Can't find illustration target!"))
            }
        }
    permission = get_edit_permission(request, content)
    if permission is None:
        raise HTTPInternalServerError("Missing permission definition!")
    if not request.has_permission(permission, context=content):
        raise HTTPForbidden()
    illustration = illustration_interface(content, None)
    if illustration is None:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'message': translate(_("No illustration on content!"))
            }
        }
    media = container[name]
    negotiator = get_utility(INegotiator)
    illustration.title = media.title
    illustration.alt_title = media.alt_title
    illustration.description = media.description
    illustration.author = media.author
    illustration.data = {
        negotiator.server_language: copy(media.data)
    }
    return {
        'status': 'success',
        'message': translate(_("Illustration has been updated successfully."))
    }


@view_config(name='set-content-illustration.json',
             context=IGalleryContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def set_content_illustration(request):
    """Set content illustration from medias gallery file"""
    return set_illustration(request, IIllustration)


@view_config(name='set-link-illustration.json',
             context=IGalleryContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def set_link_illustration(request):
    """Set link illustration from medias gallery file"""
    return set_illustration(request, ILinkIllustration)


override_template(GalleryMediasViewlet,
                  template='templates/gallery-medias.pt',
                  layer=IAdminLayer)

override_template(GalleryView,
                  template='templates/gallery-view.pt',
                  layer=IAdminLayer)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from persistent import Persistent
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.component.address.interfaces import IAddress
from pyams_utils.factory import factory_config


__docformat__ = 'restructuredtext'


@factory_config(IAddress)
class Address(Persistent):
    """Address persistent class"""

    street = FieldProperty(IAddress['street'])
    locality = FieldProperty(IAddress['locality'])
    postal_code = FieldProperty(IAddress['postal_code'])
    city = FieldProperty(IAddress['city'])
    comments = FieldProperty(IAddress['comments'])



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from datetime import time

from zope.annotation import IAttributeAnnotatable
from zope.container.constraints import contains
from zope.container.interfaces import IContainer
from zope.schema import Bool, Int, Text, TextLine, Time

from pyams_app_msc import _


ROOMS_VOCABULARY = 'msc.theater.rooms'


class ICinemaRoom(IAttributeAnnotatable):
    """Cinema room interface"""

    active = Bool(title=_("Active room?"),
                  description=_("An inactive room can't be selected to assign new activities"),
                  required=True,
                  default=True)

    name = TextLine(title=_("Room name or number"),
                    description=_("This name should be unique inside a single theater..."),
                    required=True)

    capacity = Int(title=_("Capacity"),
                   description=_("Maximum number or seats available in this room"),
                   required=True,
                   min=0)

    start_time = Time(title=_("Opening time"),
                      description=_("First time available for sessions planning"),
                      required=True,
                      default=time(hour=8))

    end_time = Time(title=_("Closing time"),
                    description=_("Last time available for sessions planning"),
                    required=True,
                    default=time(hour=23, minute=59))

    notepad = Text(title=_("Notepad"),
                   description=_("This comment is for internal use only"),
                   required=False)


CINEMA_ROOM_CONTAINER_KEY = 'msc.room.container'


class ICinemaRoomContainer(IContainer):
    """Cinema room container interface"""

    contains(ICinemaRoom)

    def append(self, item):
        """Add room to container"""

    def get_active_items(self):
        """Get iterator over active items"""


class ICinemaRoomContainerTarget(IAttributeAnnotatable):
    """Cinema room container target marker interface"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_msc.feature.booking.zmi.dashboard module

"""

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Any, Eq
from zope.dublincore.interfaces import IZopeDublinCore
from zope.interface import Attribute, Interface, implementer
from zope.schema.vocabulary import getVocabularyRegistry

from pyams_app_msc.feature.booking.interfaces import BOOKING_STATUS, BOOKING_STATUS_LABEL, IBookingInfo
from pyams_app_msc.feature.booking.zmi.interfaces import IBookingAcceptedStatusTable, IBookingDashboardMenu, \
    IBookingManagementMenu, IBookingStatusTable, IBookingWaitingStatusTable
from pyams_app_msc.feature.profile.interfaces import IUserProfile
from pyams_app_msc.interfaces import VIEW_BOOKING_PERMISSION
from pyams_app_msc.shared.catalog.interfaces import ICatalogEntry
from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_app_msc.shared.theater.interfaces.room import ROOMS_VOCABULARY, ICinemaRoomContainer
from pyams_catalog.query import CatalogResultSet
from pyams_content.shared.common.zmi.dashboard import BaseDashboardTable, BaseSharedToolDashboardView
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.utility import get_principal
from pyams_table.column import GetAttrColumn
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.date import SH_DATETIME_FORMAT, format_datetime
from pyams_utils.dict import DotDict
from pyams_utils.factory import factory_config, get_interface_base_name
from pyams_utils.interfaces import ICacheKeyValue, MISSING_INFO
from pyams_utils.list import unique_iter
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.manager import viewletmanager_config
from pyams_workflow.interfaces import IWorkflowVersions
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import IInnerTable, ITableElementEditor, ITableWithActions
from pyams_zmi.interfaces.viewlet import IMenuHeader, INavigationViewletManager
from pyams_zmi.table import I18nColumnMixin, IconColumn, MultipleTablesAdminView, TableElementEditor
from pyams_zmi.utils import get_object_label
from pyams_zmi.zmi.viewlet.menu import NavigationMenu, NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewletmanager_config(name='booking-manager.menu',
                       context=IMovieTheater, layer=IAdminLayer,
                       manager=INavigationViewletManager, weight=50,
                       provides=IBookingManagementMenu)
class BookingManagementMenu(NavigationMenu):
    """Booking management menu"""

    _header = _("Booking management")


@adapter_config(required=(IMovieTheater, IAdminLayer, Interface, IBookingManagementMenu),
                provides=IMenuHeader)
def movie_theater_booking_manager_menu_header(context, request, view, menu):
    """Movie theater booking manager menu header"""
    return request.localizer.translate(_("Booking management"))


@viewletmanager_config(name='booking-dashboard.menu',
                       context=IMovieTheater, layer=IAdminLayer,
                       manager=IBookingManagementMenu, weight=5,
                       provides=IBookingDashboardMenu,
                       permission=VIEW_BOOKING_PERMISSION)
class BookingDashboardMenu(NavigationMenuItem):
    """Movie theater booking dashboard menu"""

    label = _("Dashboard")
    icon_class = 'fas fa-chart-line'
    href = '#booking-dashboard.html'


@pagelet_config(name='booking-dashboard.html',
                context=IMovieTheater, layer=IPyAMSLayer,
                permission=VIEW_BOOKING_PERMISSION)
class BookingDashboardView(MultipleTablesAdminView):
    """Booking dashboard view"""

    header_label = _("Booking dashboard")
    table_label = _("Booking dashboard")


#
# Generic booking tables components
#

class IBookingElement(Interface):
    """Booking info interface"""
    booking = Attribute("Booking getter")
    session = Attribute("Session getter")
    entry = Attribute("Catalog entry getter")


@implementer(IBookingElement)
class BookingElement(DotDict):
    """Booking element class"""

    @property
    def __name__(self):
        return self.booking.__name__

    @property
    def __parent__(self):
        return self.booking.__parent__

    @property
    def __acl__(self):
        return self.booking.__acl__


def get_booking_element(booking):
    """Get booking element from booking input"""
    session = booking.session
    wf_entry = None
    entry = get_parent(session, ICatalogEntry)
    if entry is not None:
        wf_entry = IWorkflowVersions(entry).get_last_versions()[0]
    return BookingElement({
        'booking': booking,
        'session': session,
        'entry': wf_entry
    })


@adapter_config(required=IBookingElement,
                provides=ICacheKeyValue)
def booking_element_cache_key_value(context):
    """Booking element cache key value adapter"""
    return ICacheKeyValue(context.booking)


@adapter_config(required=IBookingElement,
                provides=IBookingInfo)
def booking_element_info(context):
    """Booking element info adapter"""
    return context.booking


@adapter_config(required=(IBookingElement, IAdminLayer, IBookingStatusTable),
                provides=ITableElementEditor)
class BookingElementEditor(TableElementEditor):
    """Booking element editor"""

    @property
    def href(self):
        return absolute_url(self.context.booking, self.request, 'properties.html')


@implementer(IBookingStatusTable, ITableWithActions)
class BookingStatusTable(BaseDashboardTable):
    """Movie theater booking status table"""

    @property
    def sort_index(self):
        return len(self.columns) - 3

    sort_order = 'asc'


@adapter_config(name='session',
                required=(IMovieTheater, IAdminLayer, IBookingStatusTable),
                provides=IColumn)
class BookingStatusSessionColumn(I18nColumnMixin, GetAttrColumn):
    """Booking status session column"""

    i18n_header = _("Session")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 10
    responsive_priority = 1

    def get_value(self, obj):
        if obj.entry is None:
            label = obj.session.label
            if label:
                label = f'({label})'
            else:
                label = self.request.localizer.translate(_("Out of catalog activity"))
            return label
        return get_object_label(obj.entry, self.request, self.table)


@adapter_config(name='room',
                required=(IMovieTheater, IAdminLayer, IBookingStatusTable),
                provides=IColumn)
class BookingStatusRoomColumn(I18nColumnMixin, GetAttrColumn):
    """Booking status room column"""

    i18n_header = _("Room")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 15

    def get_value(self, obj):
        room = ICinemaRoomContainer(self.context).get(obj.session.room)
        return get_object_label(room, self.request, self.table) if room is not None else MISSING_INFO


@adapter_config(name='date',
                required=(IMovieTheater, IAdminLayer, IBookingStatusTable),
                provides=IColumn)
class BookingStatusDateColumn(I18nColumnMixin, GetAttrColumn):
    """Booking status date column"""

    i18n_header = _("Date")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 20

    def get_value(self, obj):
        return format_datetime(obj.session.start_date, SH_DATETIME_FORMAT)


@adapter_config(name='creator',
                required=(IMovieTheater, IAdminLayer, IBookingStatusTable),
                provides=IColumn)
class BookingStatusCreatorColumn(I18nColumnMixin, GetAttrColumn):
    """Booking status creator column"""

    i18n_header = _("Creator")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 23

    def get_value(self, obj):
        return get_principal(self.request, obj.booking.creator).title


@adapter_config(name='recipient',
                required=(IMovieTheater, IAdminLayer, IBookingStatusTable),
                provides=IColumn)
class BookingStatusRecipientColumn(I18nColumnMixin, GetAttrColumn):
    """Booking status recipient column"""

    i18n_header = _("Recipient")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 25

    def get_value(self, obj):
        recipient = get_principal(self.request, obj.booking.recipient)
        result = recipient.title
        profile_info = IUserProfile(recipient, None)
        if (profile_info is None) or not profile_info.establishment:
            return result
        result = f'{result} ({profile_info.establishment})'
        if obj.booking.cultural_pass:
            label = self.request.localizer.translate(_("Cultural pass"))
            result += (f' <img title="{label}" alt="" '
                       f'      class="mx-1 mt-n1 hint" '
                       f'      src="/--static--/msc/img/pass-culture.webp" '
                       f'      width="24" height="24" />')
        return result


@adapter_config(name='seats',
                required=(IMovieTheater, IAdminLayer, IBookingStatusTable),
                provides=IColumn)
class BookingStatusSeatsColumn(I18nColumnMixin, GetAttrColumn):
    """Booking status seats column"""

    i18n_header = _("Seats")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 30

    def get_value(self, obj):
        return f'{obj.booking.nb_participants} + {obj.booking.nb_accompanists}'


@adapter_config(name='status',
                required=(IMovieTheater, IAdminLayer, IBookingStatusTable),
                provides=IColumn)
class BookingStatusStatusColumn(I18nColumnMixin, GetAttrColumn):
    """Booking status column"""

    i18n_header = _("Status")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 35

    def get_value(self, obj):
        translate = self.request.localizer.translate
        return translate(BOOKING_STATUS_LABEL.get(BOOKING_STATUS(obj.booking.status)))


@adapter_config(name='quotation',
                required=(IMovieTheater, IAdminLayer, IBookingStatusTable),
                provides=IColumn)
class BookingStatusQuotationColumn(IconColumn):
    """Booking status quotation column"""

    icon_class = 'fas fa-file-pdf'
    hint = _("Quotation")
    weight = 40

    @staticmethod
    def checker(obj):
        return (obj.booking.status == BOOKING_STATUS.ACCEPTED.value) and obj.booking.quotation

    def render_cell(self, item):
        result = super().render_cell(item)
        if result:
            quotation = item.booking.quotation
            return f'<a href="{absolute_url(quotation, self.request)}" target="_blank">{result}</a>'
        return result


@adapter_config(name='timestamp',
                required=(IMovieTheater, IAdminLayer, IBookingStatusTable),
                provides=IColumn)
class BookingStatusTimestampColumn(I18nColumnMixin, GetAttrColumn):
    """Booking status timestamp column"""

    i18n_header = _("Last update")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 90

    def get_value(self, obj):
        booking_dc = IZopeDublinCore(obj.booking)
        return format_datetime(booking_dc.modified, SH_DATETIME_FORMAT)


@adapter_config(name='archived',
                required=(IMovieTheater, IAdminLayer, IBookingStatusTable),
                provides=IColumn)
class BookingStatusArchivedColumn(IconColumn):
    """Booking status archived column"""

    css_classes = {
        'th': 'action',
        'td': 'text-danger'
    }
    icon_class = 'fas fa-archive'
    hint = _("Archived")
    weight = 100

    @staticmethod
    def checker(obj):
        return obj.booking.archived


#
# Waiting bookings table
#

@factory_config(IBookingWaitingStatusTable)
class BookingWaitingStatusTable(BookingStatusTable):
    """Booking waiting status table"""

    prefix = 'waiting_table'


@adapter_config(required=(IMovieTheater, IAdminLayer, BookingWaitingStatusTable),
                provides=IValues)
class BookingWaitingStatusTableValues(ContextRequestViewAdapter):
    """Movie theater booking waiting status table values"""

    @property
    def values(self):
        """Table values getter"""
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, ROOMS_VOCABULARY)
        params = And(Eq(catalog['object_types'], get_interface_base_name(IBookingInfo)),
                     Any(catalog['planning_room'], vocabulary.by_value.keys()),
                     Any(catalog['booking_status'],
                         {BOOKING_STATUS.OPTION.value, BOOKING_STATUS.WAITING.value}))
        yield from map(get_booking_element,
                       filter(lambda x: not x.archived,
                              unique_iter(CatalogResultSet(CatalogQuery(catalog).query(params)))))


@adapter_config(name='booking-waiting-status',
                required=(IMovieTheater, IAdminLayer, BookingDashboardView),
                provides=IInnerTable)
class BookingWaitingStatusView(BaseSharedToolDashboardView):
    """Movie theater booking waiting status view"""

    table_class = BookingWaitingStatusTable

    empty_label = _("MANAGER - No booking waiting for your action")
    single_label = _("MANAGER - 1 booking waiting for your action")
    plural_label = _("MANAGER - {} bookings waiting for your action")

    weight = 10


#
# Accepted bookings table
#

@factory_config(IBookingAcceptedStatusTable)
class BookingAcceptedStatusTable(BookingStatusTable):
    """Booking accepted status table"""

    prefix = 'accepted_table'


@adapter_config(required=(IMovieTheater, IAdminLayer, BookingAcceptedStatusTable),
                provides=IValues)
class BookingAcceptedStatusTableValues(ContextRequestViewAdapter):
    """Movie theater booking accepted status table values"""

    @property
    def values(self):
        """Table values getter"""
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, ROOMS_VOCABULARY)
        params = And(Eq(catalog['object_types'], get_interface_base_name(IBookingInfo)),
                     Any(catalog['planning_room'], vocabulary.by_value.keys()),
                     Eq(catalog['booking_status'], BOOKING_STATUS.ACCEPTED.value))
        yield from map(get_booking_element,
                       filter(lambda x: not x.archived,
                              unique_iter(CatalogResultSet(CatalogQuery(catalog).query(params)))))


@adapter_config(name='booking-accepted-status',
                required=(IMovieTheater, IAdminLayer, BookingDashboardView),
                provides=IInnerTable)
class BookingAcceptedStatusView(BaseSharedToolDashboardView):
    """Movie theater booking accepted status view"""

    table_class = BookingAcceptedStatusTable

    empty_label = _("MANAGER - No booking accepted")
    single_label = _("MANAGER - 1 booking accepted")
    plural_label = _("MANAGER - {} bookings accepted")

    weight = 20



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from persistent import Persistent
from pyramid_mailer.interfaces import IMailer
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.feature.messaging.interfaces import IMessagingSettings, MESSAGING_SETTINGS_KEY
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config

__docformat__ = 'restructuredtext'

from pyams_utils.registry import query_utility


@factory_config(IMessagingSettings)
class MessagingSettings(Persistent, Contained):
    """Messaging settings persistent class"""

    mailer_name = FieldProperty(IMessagingSettings['mailer_name'])
    source_name = FieldProperty(IMessagingSettings['source_name'])
    source_address = FieldProperty(IMessagingSettings['source_address'])
    subject_prefix = FieldProperty(IMessagingSettings['subject_prefix'])

    def get_mailer(self):
        """Mailer utility getter"""
        return query_utility(IMailer, name=self.mailer_name)


@adapter_config(required=ISiteRoot,
                provides=IMessagingSettings)
def messaging_settings(context):
    """Messaging settings adapter"""
    return get_annotation_adapter(context, MESSAGING_SETTINGS_KEY, IMessagingSettings)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface

from pyams_app_msc.interfaces import MANAGE_THEATER_PERMISSION
from pyams_app_msc.shared.theater import IMailTemplatesTarget
from pyams_app_msc.shared.theater.interfaces.mail import IMailTemplates
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces.form import IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_skin.interfaces.viewlet import IFormHeaderViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewlet_config(name='mail-templates.menu',
                context=IMailTemplatesTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=590,
                permission=MANAGE_THEATER_PERMISSION)
class MailTemplatesMenu(NavigationMenuItem):
    """Mail templates menu"""

    label = _("Mail templates")
    href = '#mail-templates.html'


@ajax_form_config(name='mail-templates.html',
                  context=IMailTemplatesTarget, layer=IPyAMSLayer,
                  permission=MANAGE_THEATER_PERMISSION)
class MailTemplatesEditForm(AdminEditForm):
    """Mail templates edit form"""

    title = _("Movie theater mail templates")
    legend = _("Mail templates")

    fields = Fields(Interface)


@adapter_config(name='main',
                context=(IMailTemplatesTarget, IAdminLayer, MailTemplatesEditForm),
                provides=IGroup)
class MailTemplatesMainGroup(Group):
    """Mail templates main group"""

    legend = _("Messages management")
    fields = Fields(IMailTemplates).select('send_copy_to_sender')

    weight = 0


@adapter_config(name='cancel',
                required=(IMailTemplatesTarget, IAdminLayer, MailTemplatesEditForm),
                provides=IGroup)
class MailTemplatesCancelGroup(Group):
    """Mail templates cancel group"""

    legend = _("Cancelled booking")
    fields = Fields(IMailTemplates).select('cancel_subject', 'cancel_template')

    weight = 10


@adapter_config(name='refuse',
                required=(IMailTemplatesTarget, IAdminLayer, MailTemplatesEditForm),
                provides=IGroup)
class MailTemplatesRefuseGroup(Group):
    """Mail templates refuse group"""

    legend = _("Refused booking")
    fields = Fields(IMailTemplates).select('refuse_subject', 'refuse_template')

    weight = 20


@adapter_config(name='option',
                required=(IMailTemplatesTarget, IAdminLayer, MailTemplatesEditForm),
                provides=IGroup)
class MailTemplatesOptionGroup(Group):
    """Mail templates option group"""

    legend = _("Booking accepted 'in option'")
    fields = Fields(IMailTemplates).select('option_subject', 'option_template')

    weight = 30


@adapter_config(name='accept',
                required=(IMailTemplatesTarget, IAdminLayer, MailTemplatesEditForm),
                provides=IGroup)
class MailTemplatesAcceptGroup(Group):
    """Mail templates accept group"""

    legend = _("Accepted booking")
    fields = Fields(IMailTemplates).select('accept_subject', 'accept_template')

    weight = 40


@adapter_config(name='update',
                required=(IMailTemplatesTarget, IAdminLayer, MailTemplatesEditForm),
                provides=IGroup)
class MailTemplatesUpdateGroup(Group):
    """Mail templates update group"""

    legend = _("Updated booking")
    fields = Fields(IMailTemplates).select('update_subject', 'update_template')

    weight = 50


@adapter_config(name='reminder',
                required=(IMailTemplatesTarget, IAdminLayer, MailTemplatesEditForm),
                provides=IGroup)
class MailTemplatesReminderGroup(Group):
    """Mail templates reminder group"""

    legend = _("Booking reminder")
    fields = Fields(IMailTemplates).select('reminder_subject', 'reminder_template')

    weight = 60


@viewlet_config(name='mail-templates.help',
                context=IMailTemplatesTarget, layer=IAdminLayer, view=MailTemplatesEditForm,
                manager=IFormHeaderViewletManager, weight=10)
class MailTemplatesEditFormHelp(AlertMessage):
    """Mail templates edit form help"""

    message_renderer = 'markdown'
    _message = _("""Mail templates can include spaces for predefined variables, including:

 - {theater_name}: theater name
 - {theater_email}: theater email address
 - {theater_phone}: theater phone number
 - {theater_logo}: theater logo image
 - {sender_name}: sender name
 - {sender_email}: sender email address
 - {booking_date}: booking date
 - {session}: session full label
 - {session_title}: session activity title
 - {session_date}: session date
 - {{session_date}}: new session date after booking update
""")



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Or
from zope.interface import Interface

from pyams_app_msc.feature.profile import IUserProfile
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_catalog.query import CatalogResultSet
from pyams_content.feature.header.portlet import IPageHeaderPortletSettings
from pyams_content.feature.header.portlet.skin import PageHeaderPortletDefaultRenderer
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_skin.viewlet.menu import MenuItem
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import get_interface_base_name
from pyams_utils.registry import get_utility, query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url

__docformat__ = 'restructuredtext'

from pyams_app_msc import _
from pyams_workflow.interfaces import IWorkflowPublicationInfo


@adapter_config(name='msc::default',
                required=(IPortalContext, IPyAMSLayer, Interface, IPageHeaderPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/header-default.pt', layer=IPyAMSLayer)
class MSCPageHeaderPortletDefaultRenderer(PageHeaderPortletDefaultRenderer):
    """Page header portlet default renderer"""

    label = _("MSC: Simple banner")

    @property
    def logo(self):
        logo = None
        theater = get_parent(self.context, IMovieTheater)
        if theater is not None:
            logo = theater.logo
        return (theater, logo) if logo else super().logo

    @property
    def admin_menus(self):
        principal_id = self.request.principal.id
        catalog = get_utility(ICatalog)
        query = And(Eq(catalog['object_types'], get_interface_base_name(IMovieTheater)),
                    Or(Eq(catalog['role:msc:manager'], principal_id),
                       Eq(catalog['role:msc:operator'], principal_id),
                       Eq(catalog['role:msc:contributor'], principal_id),
                       Eq(catalog['role:msc:reader'], principal_id)))
        for theater in sorted(CatalogResultSet(CatalogQuery(catalog).query(query)),
                              key=lambda x: II18n(x).query_attribute('title', request=self.request)):
            menu = MenuItem(theater, self.request, self, None)
            menu.label = II18n(theater).query_attribute('title', request=self.request)
            menu.href = absolute_url(theater, self.request, 'admin')
            menu.update()
            yield menu
        else:
            profile = IUserProfile(self.request, None)
            if profile is not None:
                for theater_name in (profile.local_theaters or()):
                    theater = query_utility(IMovieTheater, name=theater_name)
                    if (theater is not None) and IWorkflowPublicationInfo(theater).is_visible(self.request):
                        menu = MenuItem(theater, self.request, self, None)
                        menu.label = II18n(theater).query_attribute('title', request=self.request)
                        menu.href = absolute_url(theater, self.request)
                        menu.update()
                        yield menu



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import implementer
from zope.schema import Int

from pyams_app_msc.component.duration.interfaces import IDurationField

__docformat__ = 'restructuredtext'


@implementer(IDurationField)
class DurationField(Int):
    """Custom duration field"""

    def __init__(self, min=None, max=None, default=None, **kwargs):
        super().__init__(min=0, max=1440, default=default, **kwargs)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface
from zope.schema import Bool, Int, TextLine

from pyams_content.feature.renderer import IRendererSettings
from pyams_content.shared.view.portlet.skin import IViewItemsPortletPanelsRendererSettings

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class ICatalogViewItemsPortletCalendarRendererSettings(Interface):
    """Catalog view items portlet calendar renderer settings"""

    filters_css_class = TextLine(title=_('Filters CSS class'),
                                 description=_("CSS class used for filters column"),
                                 default='col col-12 col-md-4 col-lg-3 col-xl-2 float-left text-md-right')

    calendar_css_class = TextLine(title=_('Calendar CSS class'),
                                  description=_("CSS class used for calendar container"),
                                  default='row mx-0 col col-12 col-md-8 col-lg-9 col-xl-10 float-right')

    sessions_weeks = Int(title=_("Sessions weeks"),
                         description=_("Number of weeks to display sessions"),
                         required=True,
                         default=4)


class ICatalogViewItemsPortletPanelsRendererSettings(IViewItemsPortletPanelsRendererSettings):
    """Catalog view items portlet panels renderers settings"""

    filters_css_class = TextLine(title=_('Filters CSS class'),
                                 description=_("CSS class used for filters column"),
                                 default='col col-12 col-md-4 col-lg-3 col-xl-2 float-left text-md-right')

    results_css_class = TextLine(title=_('Results CSS class'),
                                 description=_("CSS class used for view items container"),
                                 default='row mx-0 col col-12 col-md-8 col-lg-9 col-xl-10 float-right')

    first_panel_css_class = TextLine(title=_('First panel CSS class'),
                                     description=_("CSS class used for first view items panel"),
                                     default='col col-12 col-sm-6 col-md-4 col-lg-3 col-xl-2 my-3 d-flex flex-column')

    panels_css_class = TextLine(title=_('Panels CSS class'),
                                description=_("CSS class used for view items panels"),
                                default='col col-6 col-md-4 col-lg-3 col-xl-2 my-3 d-flex flex-column')

    display_sessions = Bool(title=_("Display sessions"),
                            description=_("If 'no', incoming sessions will not be displayed"),
                            required=True,
                            default=True)

    sessions_weeks = Int(title=_("Sessions weeks"),
                         description=_("Number of weeks to display sessions"),
                         required=True,
                         default=4)

    display_free_seats = Bool(title=_("Display free seats"),
                              description=_("If 'no', number of free seats is not displayed"),
                              required=True,
                              default=True)


class ICatalogEntrySpecificitiesPortletRendererSettings(IRendererSettings):
    """Catalog entry specificities portlet renderer settings interface"""

    display_sessions = Bool(title=_("Display sessions"),
                            description=_("If 'no', incoming sessions will not be displayed"),
                            required=True,
                            default=True)

    sessions_weeks = Int(title=_("Sessions weeks"),
                         description=_("Number of weeks to display sessions"),
                         required=True,
                         default=4)

    display_free_seats = Bool(title=_("Display free seats"),
                              description=_("If 'no', number of free seats is not displayed"),
                              required=True,
                              default=True)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""
from datetime import datetime, timezone

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Ge, Le, Not, Or
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_app_msc.shared.theater.portlet.interfaces import IMovieTheaterCarouselPortletSettings, THEATERS_ORDER
from pyams_catalog.query import CatalogResultSet, IsNone
from pyams_i18n.interfaces import II18n
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_utils.factory import factory_config, get_interface_base_name
from pyams_utils.list import random_iter
from pyams_utils.registry import get_utility
from pyams_utils.request import query_request

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


MOVIE_THEATER_CAROUSEL_PORTLET_NAME = 'msc.portlet.theater.carousel'
MOVIE_THEATER_CAROUSEL_ICON_CLASS = 'far fa-images'


@factory_config(IMovieTheaterCarouselPortletSettings)
class MovieTheaterCarouselPortletSettings(PortletSettings):
    """Movie theater carousel portlet settings"""

    title = FieldProperty(IMovieTheaterCarouselPortletSettings['title'])
    theaters_order = FieldProperty(IMovieTheaterCarouselPortletSettings['theaters_order'])

    @property
    def theaters(self):
        """Movie theaters iterator"""
        catalog = get_utility(ICatalog)
        now= datetime.now(timezone.utc)
        query = And(Eq(catalog['object_types'], get_interface_base_name(IMovieTheater)),
                    Le(catalog['effective_date'], now),
                    Or(IsNone(catalog['expiration_date']),
                       Not(Ge(catalog['expiration_date'], now))))
        if self.theaters_order == THEATERS_ORDER.RANDOM.value:
            initial_set = CatalogResultSet(CatalogQuery(catalog).query(query))
            results_set = random_iter(initial_set, limit=len(initial_set))
        else:
            request = query_request()
            results_set = sorted(CatalogResultSet(CatalogQuery(catalog).query(query)),
                                 key=lambda x: II18n(x).query_attribute('title', request=request))
        yield from results_set


@portlet_config(permission=None)
class MovieTheaterCarouselPortlet(Portlet):
    """Movie theater carousel portlet"""

    name = MOVIE_THEATER_CAROUSEL_PORTLET_NAME
    label = _("MSC: Movie theaters list")

    settings_factory = IMovieTheaterCarouselPortletSettings
    toolbar_css_class = MOVIE_THEATER_CAROUSEL_ICON_CLASS



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.annotation.interfaces import IAttributeAnnotatable
from zope.interface import Interface
from zope.schema import TextLine, Bool

from pyams_utils.schema import HTMLField

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


MAIL_TEMPLATES_KEY = 'msc.mail.templates'


class IMailTemplates(Interface):
    """Movie theater mail templates interfaces"""

    send_copy_to_sender = Bool(title=_("Send copy to operator"),
                               description=_("If 'yes', a copy of each message will be sent to operator"),
                               required=True,
                               default=False)

    cancel_subject = TextLine(title=_("Message subject"),
                              description=_("Default subject used for cancelled booking notification"),
                              required=False)

    cancel_template = HTMLField(title=_("Message template"),
                                description=_("This template is used when a session booking is cancelled"),
                                required=False)

    refuse_subject = TextLine(title=_("Message subject"),
                              description=_("Default subject used for refused booking notification"),
                              required=False)

    refuse_template = HTMLField(title=_("Message template"),
                                description=_("This template is used when a session booking is refused"),
                                required=False)

    option_subject = TextLine(title=_("Message subject"),
                              description=_("Default subject used for temporarily accepted booking"),
                              required=False)

    option_template = HTMLField(title=_("Message template"),
                                description=_("This template is used when a session booking is temporarily "
                                              "accepted but waiting for confirmation"),
                                required=False)

    accept_subject = TextLine(title=_("Message subject"),
                              description=_("Default subject used for accepted booking notification"),
                              required=False)

    accept_template = HTMLField(title=_("Message template"),
                                description=_("This template is used when a session booking is accepted"),
                                required=False)

    update_subject = TextLine(title=_("Message subject"),
                              description=_("Default subject used for updated booking notification"),
                              required=False)

    update_template = HTMLField(title=_("Message template"),
                                description=_("This template is used when a session booking is updated after "
                                              "being accepted"),
                                required=False)

    reminder_subject = TextLine(title=_("Reminder subject"),
                                description=_("Default subject used for reminder message"),
                                required=False)

    reminder_template = HTMLField(title=_("Message template"),
                                  description=_("This template is used for reminder messages which are "
                                                "sent to sessions recipients a few days before they take place"),
                                  required=False)


class IMailTemplatesTarget(IAttributeAnnotatable):
    """Cinema price container target marker interface"""



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.shared.catalog.interfaces import IAudienceFilter, IDurationFilter
from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_app_msc.shared.theater.interfaces.audience import ICinemaAudienceContainer
from pyams_content.feature.filter import Filter
from pyams_content.feature.filter.interfaces import IAggregatedPortletRendererSettings, IFilterAggregate, \
    IFilterProcessor
from pyams_content_es.filter import get_sorting_params
from pyams_content_es.filter.processor import EsBaseFilterProcessor
from pyams_layer.interfaces import IPyAMSLayer
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_pyramid_registry
from pyams_utils.request import query_request
from pyams_utils.traversing import get_parent

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


#
# Activity audience filter
#

@factory_config(IAudienceFilter)
class AudienceFilter(Filter):
    """Audience filter"""

    filter_type = 'audience'


@adapter_config(required=IAudienceFilter,
                provides=IFilterAggregate)
def audience_filter_aggregate(context):
    """Audience filter aggregate"""
    registry = get_pyramid_registry()
    return {
        'name': context.filter_type,
        'type': 'terms',
        'params': {
            'field': registry.settings.get('pyams_content_es.filter.audience.field_name',
                                           'catalog_info.audiences'),
            'size': 100,
            'order': get_sorting_params(context.sorting_mode)
        }
    }


@adapter_config(required=(IAudienceFilter, IPyAMSLayer, IAggregatedPortletRendererSettings),
                provides=IFilterProcessor)
class AudienceFilterProcessor(EsBaseFilterProcessor):
    """Audience filter processor"""

    def convert_aggregations(self, aggregations):
        result = []
        request = query_request()
        theater = get_parent(request.context, IMovieTheater)
        if theater is None:
            return result
        audiences = ICinemaAudienceContainer(theater, {})
        for item in aggregations:
            audience = audiences.get(item['key'])
            if audience is not None:
                result.append({
                    'key': audience.__name__,
                    'label': audience.name,
                    'doc_count': item['doc_count']
                })
        return result


#
# Activity duration filter
#

DURATION_FILTER_LABEL = {
    '-30': _("< 30 min."),
    '30-60': _("30 to 60 min."),
    '60-90': _("60 to 90 min."),
    '90-': _("> 90 min.")
}


@factory_config(IDurationFilter)
class DurationFilter(Filter):
    """Duration filter"""

    filter_type = 'duration'
    sorting_mode = FieldProperty(IDurationFilter['sorting_mode'])


@adapter_config(required=IDurationFilter,
                provides=IFilterAggregate)
def duration_filter_aggregate(context):
    """Duration filter aggregate"""
    registry = get_pyramid_registry()
    return {
        'name': context.filter_type,
        'type': 'range',
        'params': {
            'field': registry.settings.get('pyams_content_es.filter.duration.field_name',
                                           'catalog_info.duration'),
            'keyed': True,
            'ranges': [{
                'key': '-30',
                'to': 30
            }, {
                'key': '30-60',
                'from': 30,
                'to': 60
            }, {
                'key': '60-90',
                'from': 60,
                'to': 90
            }, {
                'key': '90-',
                'from': 90
            }]
        }
    }


@adapter_config(required=(IDurationFilter, IPyAMSLayer, IAggregatedPortletRendererSettings),
                provides=IFilterProcessor)
class DurationFilterProcessor(EsBaseFilterProcessor):
    """Duration filter processor"""

    def convert_aggregations(self, aggregations):
        result = []
        for key in aggregations:
            result.append({
                'key': key,
                'label': self.request.localizer.translate(DURATION_FILTER_LABEL[key]),
                'doc_count': aggregations.buckets[key].doc_count
            })
        return result



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.httpexceptions import HTTPNotFound
from zope.container.btree import BTreeContainer
from zope.interface import classImplements
from zope.location.interfaces import ISublocations
from zope.traversing.interfaces import ITraversable
from ZODB.interfaces import IConnection

from pyams_app_msc.feature.booking import BOOKING_STATUS
from pyams_app_msc.feature.booking.interfaces import BOOKING_CONTAINER_KEY, IBookingContainer, \
    IBookingTarget, OCCUPIED_BOOKING_STATUS, REQUESTED_BOOKING_STATUS
from pyams_app_msc.feature.planning.interfaces import ISession
from pyams_app_msc.feature.planning.session import Session
from pyams_app_msc.feature.profile.interfaces import SEATS_DISPLAY_MODE
from pyams_catalog.utils import index_object
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.traversing import get_parent
from pyams_utils.zodb import volatile_property

__docformat__ = 'restructuredtext'


@factory_config(IBookingContainer)
class BookingContainer(BTreeContainer):
    """Booking container"""

    @volatile_property
    def session(self):
        """Booking session getter"""
        return get_parent(self, IBookingTarget)

    def append(self, booking):
        """Add booking to container"""
        IConnection(self).add(booking)
        key = ICacheKeyValue(booking)
        self[key] = booking
        index_object(booking)

    def get_requested_seats(self):
        """Get number of requested seats"""
        return sum((
            booking.nb_seats
            for booking in self.values()
            if booking.status in REQUESTED_BOOKING_STATUS
        ))

    def get_waiting_seats(self):
        """Get number of waiting seats"""
        return sum((
            booking.nb_seats
            for booking in self.values()
            if booking.status == BOOKING_STATUS.WAITING.value
        ))

    def get_confirmed_seats(self):
        """Get number of confirmed reserved seats"""
        return sum((
            booking.nb_seats
            for booking in self.values()
            if booking.status in OCCUPIED_BOOKING_STATUS
        ))

    @property
    def free_seats(self):
        """Free seats getter"""
        return (self.session.capacity or 0) - self.get_confirmed_seats()

    def get_seats(self, display_mode):
        """Get total number of seats according to given display mode"""
        if display_mode == SEATS_DISPLAY_MODE.TOTAL.value:
            return (f'{self.get_confirmed_seats()} / '
                    f'{self.get_requested_seats()} / '
                    f'{self.session.capacity}')
        if display_mode == SEATS_DISPLAY_MODE.WAITING.value:
            return (f'{self.get_confirmed_seats()} / '
                    f'{self.get_waiting_seats()} / '
                    f'{self.session.capacity}')
        return (f'{self.get_confirmed_seats()} / '
                f'{self.session.capacity}')


@adapter_config(required=IBookingTarget,
                provides=IBookingContainer)
def session_booking_container(context):
    """Session booking container adapter"""
    return get_annotation_adapter(context, BOOKING_CONTAINER_KEY, IBookingContainer,
                                  name='++booking++')


@adapter_config(required=IBookingContainer,
                provides=ISession)
def booking_container_session(context):
    """Booking container session adapter"""
    return context.session


@adapter_config(name='booking',
                required=IBookingTarget,
                provides=ITraversable)
class BookingTraverser(ContextAdapter):
    """Booking target traverser"""

    def traverse(self, name, furtherPath=None):
        """Traverse to inner bookings container"""
        container = IBookingContainer(self.context, None)
        if container is not None:
            return container
        raise HTTPNotFound


@adapter_config(name='booking',
                required=IBookingTarget,
                provides=ISublocations)
class BookingSublocations(ContextAdapter):
    """Booking sub-locations"""

    def sublocations(self):
        """Booking target sub-locations getter"""
        container = IBookingContainer(self.context, None)
        if container is not None:
            yield from container.values()


classImplements(Session, IBookingTarget)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface, alsoProvides, implementer
from zope.schema.vocabulary import SimpleTerm

from pyams_app_msc.feature.tmdb import ITMDBService
from pyams_app_msc.feature.tmdb.interfaces import TMDB_SEARCH_PATH, TMDB_SEARCH_ROUTE
from pyams_app_msc.feature.tmdb.zmi.lookup import ITMDBSearchInterface
from pyams_app_msc.interfaces import MANAGE_CATALOG_PERMISSION
from pyams_app_msc.shared.catalog.interfaces import ICatalogEntryInfo, ICatalogManager, ICatalogManagerTarget, \
    IWfCatalogEntry, IWfCatalogEntryAddInfo
from pyams_app_msc.shared.catalog.zmi.interfaces import ICatalogManagementMenu, ICatalogManagementView
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_content.shared.common.zmi import SharedContentAddAction, SharedContentAddForm
from pyams_content.shared.common.zmi.content import SharedContentHeaderViewlet
from pyams_content.shared.common.zmi.types.content import TypedSharedContentCustomInfoEditForm, \
    TypedSharedContentPropertiesEditForm
from pyams_form.ajax import AJAXFormRenderer, ajax_form_config
from pyams_form.browser.select import SelectWidget
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IFormContent
from pyams_form.widget import FieldWidget
from pyams_i18n.interfaces import INegotiator
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.viewlet import IHeaderViewletManager
from pyams_skin.interfaces.widget import IDynamicSelectWidget, IHTMLEditorConfiguration
from pyams_utils.adapter import NullAdapter, adapter_config
from pyams_utils.factory import create_object
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu, IToolbarViewletManager
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@implementer(IDynamicSelectWidget)
class TMDBMovieSelectWidget(SelectWidget):
    """TMDB movie select widget"""

    placeholder = _("Enter text for TMDB movie search")

    @property
    def ajax_url(self):
        """AJAX search API URL getter"""
        return self.request.registry.settings.get(f'{TMDB_SEARCH_ROUTE}_route.path',
                                                  TMDB_SEARCH_PATH)

    @staticmethod
    def term_factory(value):
        """Select movie term factory"""
        return SimpleTerm(value)


def TMDBMovieFieldSelectWidget(field, request):
    """TMDB movie field select widget"""
    return FieldWidget(field, TMDBMovieSelectWidget(request))


@viewlet_config(name='add-content.action',
                context=IMovieTheater, layer=IAdminLayer)
class MovieTheaterSharedContentAddAction(NullAdapter):
    """Disabled shared content add action on movie theater"""


@viewlet_config(name='add-content.action',
                context=ICatalogManagerTarget, layer=IAdminLayer,
                view=ICatalogManagementView,
                manager=IToolbarViewletManager, weight=10,
                permission=MANAGE_CATALOG_PERMISSION)
class CatalogEntryAddAction(SharedContentAddAction):
    """Catalog entry add action"""

    label = _("Add catalog entry")


@ajax_form_config(name='add-shared-content.html',
                  context=ICatalogManagerTarget, layer=IPyAMSLayer,
                  permission=MANAGE_CATALOG_PERMISSION)
class CatalogEntryAddForm(SharedContentAddForm):
    """Catalog entry add form"""

    legend = _("New catalog entry properties")

    fields = Fields(IWfCatalogEntryAddInfo) + \
        Fields(IWfCatalogEntry).select('data_type') + \
        Fields(ITMDBSearchInterface) + \
        Fields(IWfCatalogEntry).select('notepad')
    fields['title'].widget_factory = TMDBMovieFieldSelectWidget

    _edit_permission = MANAGE_CATALOG_PERMISSION

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        title = self.widgets.get('title')
        if title is not None:
            title.object_data = {
                'ams-select2-options': {
                    'tags': True
                }
            }
            alsoProvides(title, IObjectData)

    @property
    def container_target(self):
        return ICatalogManager(self.context)

    def update_content(self, obj, data):
        negotiator = get_utility(INegotiator)
        title = data.get(self, data).pop('title', '')
        if title.startswith('movie_id::'):
            service = create_object(ITMDBService)
            if (service is not None) and (service.configuration is not None):
                _, movie_id = title.split('::', 1)
                movie = service.get_movie_info(movie_id, with_credits=False)
                if movie is not None:
                    obj.title = {negotiator.server_language: movie.get('title')}
                    obj.tmdb_movie_id = movie.get('id')
        else:
            obj.title = {negotiator.server_language: title}
        return super().update_content(obj, data)


@adapter_config(required=(ICatalogManagerTarget, IAdminLayer, CatalogEntryAddForm),
                provides=IAJAXFormRenderer)
class CatalogEntryAddFormRenderer(AJAXFormRenderer):
    """Catalog entry add form renderer"""

    def render(self, changes):
        return {
            'status': 'redirect',
            'location': self.form.next_url()
        }


@viewlet_config(name='pyams.content_header',
                context=IWfCatalogEntry, layer=IAdminLayer,
                manager=IHeaderViewletManager, weight=10)
class CatalogEntryHeaderViewlet(SharedContentHeaderViewlet):
    """Catalog entry header viewlet"""

    @property
    def parent_target_url(self):
        """Parent target URL"""
        tool = get_parent(self.context, IMovieTheater)
        if tool is None:
            return None
        return absolute_url(tool, self.request, 'admin#dashboard.html')


@ajax_form_config(name='properties.html',
                  context=IWfCatalogEntry, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class CatalogEntryPropertiesEditForm(TypedSharedContentPropertiesEditForm):
    """Catalog entry properties edit form"""

    interface = IWfCatalogEntry
    fieldnames = ('title', 'short_name', 'content_url', 'data_type',
                  'audiences', 'allow_session_request', 'header',
                  'description', 'notepad')


@viewlet_config(name='activity-info.menu',
                context=IWfCatalogEntry, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=15,
                permission=VIEW_SYSTEM_PERMISSION)
class CatalogEntryInformationMenu(NavigationMenuItem):
    """Activity information menu"""

    label = _("Activity details")
    href = '#activity-info.html'


@ajax_form_config(name='activity-info.html',
                  context=IWfCatalogEntry, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class CatalogEntryInformationEditForm(TypedSharedContentCustomInfoEditForm):
    """Activity information edit form"""

    title = _("Activity information")
    legend = _("Custom properties")

    @property
    def fields(self):
        """Form fields getter"""
        datatype = self.datatype
        if datatype is None:
            return Fields(Interface)
        return Fields(ICatalogEntryInfo).select(*datatype.field_names)

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        synopsis = self.widgets.get('synopsis')
        if synopsis is not None:
            synopsis.add_class('h-200px')


@adapter_config(required=(IWfCatalogEntry, IPyAMSLayer, CatalogEntryInformationEditForm),
                provides=IFormContent)
def catalog_entry_info_edit_form_content(context, request, form):
    """Catalog entry information edit form content getter"""
    return ICatalogEntryInfo(context)


@adapter_config(required=(ICatalogEntryInfo, IAdminLayer, CatalogEntryInformationEditForm),
                provides=IViewContextPermissionChecker)
def catalog_entry_info_permission_checker(context, request, view):
    """Catalog entry activity info permission checker"""
    catalog_entry = get_parent(context, IWfCatalogEntry)
    return IViewContextPermissionChecker(catalog_entry)


@adapter_config(name='description',
                required=(ICatalogEntryInfo, IAdminLayer, CatalogEntryInformationEditForm),
                provides=IHTMLEditorConfiguration)
def catalog_entry_description_editor_configuration(context, request, view):
    """Catalog entry description editor configuration"""
    return {
        'menubar': False,
        'plugins': 'paste textcolor lists charmap link pyams_link',
        'toolbar': 'undo redo | pastetext | bold italic superscript | '
                   'forecolor backcolor | bullist numlist | '
                   'charmap pyams_link link',
        'toolbar1': False,
        'toolbar2': False,
        'height': 200
    }



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.renderers import render
from zope.interface import Interface
from zope.schema import Bool, Choice, Datetime, Int, Text, TextLine

from pyams_app_msc.feature.messaging import IMessagingSettings
from pyams_app_msc.interfaces import CREATE_BOOKING_PERMISSION, MSC_MANAGER_ROLE, MSC_OPERATOR_ROLE
from pyams_app_msc.shared.catalog import IWfCatalogEntry
from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_app_msc.shared.theater.interfaces.audience import AUDIENCES_VOCABULARY, ICinemaAudienceContainer
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.form import AddForm
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_mail.interfaces import IPrincipalMailInfo
from pyams_mail.message import HTMLMessage
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.skin.page import PortalContextIndexPage
from pyams_security.interfaces import IProtectedObject, ISecurityManager
from pyams_security.interfaces.notification import INotificationSettings
from pyams_skin.interfaces.viewlet import IFormHeaderViewletManager
from pyams_skin.schema.button import SubmitButton
from pyams_skin.viewlet.help import AlertMessage
from pyams_template.template import template_config
from pyams_utils.adapter import ContextAdapter, ContextRequestViewAdapter, adapter_config
from pyams_utils.dict import DotDict
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import canonical_url
from pyams_viewlet.viewlet import viewlet_config

try:
    from pyams_chat.interfaces import IChatMessage, IChatMessageHandler
    from pyams_chat.message import ChatMessage
except ImportError:
    ChatMessage = None

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class ISessionRequestFormInfo(Interface):
    """Session request form interface"""

    movie_title = TextLine(title=_("Movie title"),
                           description=_("Title of the movie or activity for which you request a new session"),
                           required=True)

    start_date_1 = Datetime(title=_("Start date of the session"),
                            description=_("First proposed date for this session"),
                            required=True)

    start_date_2 = Datetime(title="",
                            description=_("Second proposed date for this session"),
                            required=False)

    start_date_3 = Datetime(title="",
                            description=_("Third proposed date for this session"),
                            required=False)

    audience = Choice(title=_("Target audience"),
                      description=_("Target audience selected for this session"),
                      vocabulary=AUDIENCES_VOCABULARY,
                      required=True)

    nb_participants = Int(title=_("Participants"),
                          description=_("Number of participants seats reserved for this session"),
                          required=True,
                          min=0)

    nb_accompanists = Int(title=_("Accompanists"),
                          description=_("Total number of accompanists seats reserved for this session"),
                          required=True,
                          min=0)

    nb_groups = Int(title=_("Groups count"),
                    description=_("Number of groups or classrooms attending this session"),
                    required=True,
                    min=1,
                    default=1)

    cultural_pass = Bool(title=_("Cultural pass"),
                         description=_("Check this option if payment is done using cultural pass"),
                         required=True,
                         default=False)

    comments = Text(title=_("Comments"),
                    description=_("You can add optional comments to your session request"),
                    required=False)

    send_confirmation = Bool(title=_("Get confirmation message?"),
                             description=_("If 'yes', a confirmation message will be sent to you to "
                                           "acknowledge the session request"),
                             required=True,
                             default=True)


class ISessionRequestFormButtons(Interface):
    """Session request form buttons interface"""

    add = SubmitButton(name='add',
                       title=_("Ask for new session"))


@ajax_form_config(name='session-request.html',
                  context=IMovieTheater, layer=IPyAMSLayer,
                  permission=CREATE_BOOKING_PERMISSION)
class SessionRequestForm(AddForm, PortalContextIndexPage):
    """Session request form"""

    legend = _("Request new session")

    fields = Fields(ISessionRequestFormInfo)
    buttons = Buttons(ISessionRequestFormButtons)

    _edit_permission = CREATE_BOOKING_PERMISSION

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        audience = self.widgets.get('audience')
        if audience is not None:
            audience.prompt = True
            audience.prompt_message = self.request.localizer.translate(_("Please select target audience..."))
            selected_audience = self.request.params.get('audience')
            if selected_audience:
                audience.value = selected_audience

    @handler(buttons['add'])
    def handle_add(self, action):
        super().handle_add(self, action)

    def create_and_add(self, data):
        data = data.get(self, data)
        request = self.request
        settings = IMessagingSettings(request.root, None)
        if settings is None:
            return None
        mailer = settings.get_mailer()
        if mailer is None:
            return None
        translate = request.localizer.translate
        # get source email
        sm = get_utility(ISecurityManager)
        source_email = None
        principal = sm.get_raw_principal(request.principal.id)
        mail_info = IPrincipalMailInfo(principal, None)
        if mail_info is not None:
            source_email = [
                f'{name} <{address}>'
                for name, address in mail_info.get_addresses()
            ]
            if len(source_email) > 0:
                source_email = source_email[0]
        # create inner message
        message = None
        theater = get_parent(self.context, IMovieTheater)
        contact_email = None
        audience = ICinemaAudienceContainer(theater).get(data.get('audience'))
        if audience is not None:
            contact = audience.contact
            if contact is not None:
                contact_email = f'{contact.name} <{contact.email_address}>' if contact.email_address else None
        if not contact_email:
            contact_email = theater.contact_email
        if contact_email:
            notifications_settings = INotificationSettings(sm)
            message_body = render('templates/session-request-message.pt',
                                  request=request,
                                  value={
                                      'settings': notifications_settings,
                                      'data': DotDict(data),
                                      'audience': audience,
                                      'sender': sm.get_principal(request.principal.id)
                                  })
            message = HTMLMessage(subject=translate(_("{} New session request"))
                                      .format(settings.subject_prefix),
                                  from_addr=f'{settings.source_name} <{settings.source_address}>',
                                  to_addr=contact_email,
                                  reply_to=source_email,
                                  html=message_body)
            mailer.send(message)
            # create acknowledge message
            if data.get('send_confirmation') and source_email:
                ack_message_body = render('templates/session-request-ack.pt',
                                          request=request,
                                          value={
                                              'settings': notifications_settings,
                                              'theater': theater,
                                              'data': data,
                                              'audience': audience
                                          })
                ack_message = HTMLMessage(subject=translate(_("{} Your session request"))
                                              .format(settings.subject_prefix),
                                          from_addr=f'{settings.source_name} <{settings.source_address}>',
                                          to_addr=source_email,
                                          html=ack_message_body)
                mailer.send(ack_message)
        self.finished_state.update({
            'movie_title': data.get('movie_title')
        })
        return message


@viewlet_config(name='session-request.help',
                context=IMovieTheater, layer=IPyAMSLayer, view=SessionRequestForm,
                manager=IFormHeaderViewletManager, weight=20)
class SessionRequestInfo(AlertMessage):
    """Session request info"""

    status = 'info'
    _message = _("If the proposed activities do not meet your needs, you can request an additional session.\n"
                 "We'll get back to you as soon as possible, depending on the films in the cinema's catalog "
                 "and available slots.")


@ajax_form_config(name='session-request.html',
                  context=IWfCatalogEntry, layer=IPyAMSLayer,
                  permission=CREATE_BOOKING_PERMISSION)
class CatalogEntrySessionRequestForm(SessionRequestForm):
    """Catalog entry session request form"""

    def get_ajax_handler(self):
        return canonical_url(self.context, self.request, self.ajax_form_handler)

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        movie_title = self.widgets.get('movie_title')
        if movie_title is not None:
            movie_title.value = II18n(self.context).query_attribute('title', request=self.request)


@viewlet_config(name='session-request.activity.link',
                context=IWfCatalogEntry, layer=IPyAMSLayer, view=CatalogEntrySessionRequestForm,
                manager=IFormHeaderViewletManager, weight=10)
class CatalogEntrySessionRequestActivityLink(AlertMessage):
    """Catalog entry session request activity link"""

    status = 'info'
    message_renderer = 'markdown'

    @property
    def message(self):
        link_url = canonical_url(self.context, self.request,
                                 query={'audience': self.request.params.get('audience')})
        link_text = self.request.localizer.translate(_("Display movie info"))
        return f'''<a href="{link_url}" target="_blank">{link_text}</a>''' + \
               ''' <i class="fa fas fa-external-link"></i>'''


@viewlet_config(name='session-request.help',
                context=IWfCatalogEntry, layer=IPyAMSLayer, view=CatalogEntrySessionRequestForm,
                manager=IFormHeaderViewletManager, weight=20)
class CatalogEntrySessionRequestInfo(SessionRequestInfo):
    """Catalog entry session request info"""

    _message = _("If the proposed sessions do not meet your needs, you can request an additional session.\n"
                 "We'll get back to you as soon as possible, depending on the cinema's available slots.")


@adapter_config(required=(IMovieTheater, IPyAMSLayer, SessionRequestForm),
                provides=IAJAXFormRenderer)
@adapter_config(required=(IWfCatalogEntry, IPyAMSLayer, SessionRequestForm),
                provides=IAJAXFormRenderer)
class SessionRequestFormRenderer(ContextRequestViewAdapter):
    """Session request form renderer"""

    def render(self, changes):
        if changes is None:
            return
        if ChatMessage is not None:
            request = self.request
            translate = request.localizer.translate
            movie_title = self.view.finished_state.get('movie_title')
            message = ChatMessage(request=request,
                                  context=changes,
                                  action='notify',
                                  category='session.request',
                                  source=request.principal,
                                  title=translate(_("New session request")),
                                  message=translate(_("{principal}: {title}")).format(
                                      principal=request.principal.title,
                                      title=movie_title))
            message.send()
        return {
            'status': 'redirect',
            'location': canonical_url(self.context, self.request, 'session-request-ok.html')
        }


@pagelet_config(name='session-request-ok.html',
                layer=IPyAMSLayer)
@template_config(template='templates/session-request-ok.pt',
                 layer=IPyAMSLayer)
class SessionRequestOKView(PortalContextIndexPage):
    """Session request acknowledge view"""


if ChatMessage is not None:

    @adapter_config(name='session.request',
                    required=IChatMessage,
                    provides=IChatMessageHandler)
    class SessionRequestChatMessageHandler(ContextAdapter):
        """Session request chat message handler"""

        def get_target(self):
            """Chat message targets getter"""
            theater = get_parent(self.context.request.context, IMovieTheater)
            if theater is None:
                return None
            protection = IProtectedObject(theater)
            principals = set()
            for role_id in (MSC_MANAGER_ROLE, MSC_OPERATOR_ROLE):
                for principal_id in protection.get_principals(role_id):
                    principals.add(principal_id)
            return {
                'principals': tuple(principals)
            }



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.view import view_config
from zope.interface import Interface

from pyams_app_msc.interfaces import MANAGE_THEATER_PERMISSION, VIEW_THEATER_PERMISSION
from pyams_app_msc.shared.theater import ICinemaAudienceContainerTarget
from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_app_msc.shared.theater.interfaces.audience import ICinemaAudience, ICinemaAudienceContainer
from pyams_app_msc.shared.theater.zmi.interfaces import ICinemaAudiencesTable
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.interfaces.view import IModalEditForm
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm, SimpleAddFormRenderer, SimpleEditFormRenderer
from pyams_zmi.helper.container import delete_container_element, switch_element_attribute
from pyams_zmi.interfaces import IAdminLayer, IObjectHint, IObjectLabel, TITLE_SPAN, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IPropertiesMenu, IToolbarViewletManager
from pyams_zmi.table import AttributeSwitcherColumn, NameColumn, ReorderColumn, SortableTable, TableAdminView, \
    TableElementEditor, TrashColumn
from pyams_zmi.utils import get_object_label
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewlet_config(name='cinema-audiences.menu',
                context=ICinemaAudienceContainerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=530,
                permission=VIEW_THEATER_PERMISSION)
class CinemaAudiencesMenu(NavigationMenuItem):
    """Cinema audiences menu"""

    label = _("Cinema audiences")
    href = '#cinema-audiences.html'


@factory_config(ICinemaAudiencesTable)
class CinemaAudiencesTable(SortableTable):
    """Cinema audiences table"""

    container_class = ICinemaAudienceContainer

    display_if_empty = True


@adapter_config(required=(ICinemaAudienceContainerTarget, IAdminLayer, ICinemaAudiencesTable),
                provides=IValues)
class CinemaAudiencesTableValues(ContextRequestViewAdapter):
    """Cinema audiences table values adapter"""

    @property
    def values(self):
        """Cinema audiences values getter"""
        yield from ICinemaAudienceContainer(self.context).values()


@adapter_config(name='reorder',
                required=(ICinemaAudienceContainerTarget, IAdminLayer, ICinemaAudiencesTable),
                provides=IColumn)
class CinemaAudiencesTableReorderColumn(ReorderColumn):
    """Cinema audiences table reorder column"""


@view_config(name='reorder.json',
             context=ICinemaAudienceContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=MANAGE_THEATER_PERMISSION)
def reorder_cinema_audience_table(request):
    """Reorder cinema audience table"""
    order = request.params.get('order').split(';')
    request.context.updateOrder(order)
    return {
        'status': 'success',
        'closeForm': False
    }


@adapter_config(name='active',
                required=(ICinemaAudienceContainerTarget, IAdminLayer, ICinemaAudiencesTable),
                provides=IColumn)
class CinemaAudiencesActiveColumn(AttributeSwitcherColumn):
    """Cinema audiences table active column"""

    attribute_name = 'active'
    attribute_switcher = 'switch-active-audience.json'

    icon_off_class = 'far fa-eye-slash text-danger'

    permission = MANAGE_THEATER_PERMISSION
    weight = 1

    def get_icon_hint(self, item):
        """Icon hint getter"""
        if self.has_permission(item):
            hint = _("Click icon to switch audience activity")
        elif item.active:
            hint = _("This audience is active")
        else:
            hint = _("This audience is not active")
        return self.request.localizer.translate(hint)


@view_config(name='switch-active-audience.json',
             context=ICinemaAudienceContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def switch_active_audience(request):
    """Switch cinema audience activity flag"""
    return switch_element_attribute(request, container_factory=ICinemaAudienceContainer)


@adapter_config(name='label',
                required=(ICinemaAudienceContainerTarget, IAdminLayer, ICinemaAudiencesTable),
                provides=IColumn)
class CinemaAudienceLabelColumn(NameColumn):
    """Cinema audiences table label column"""

    attr_name = 'name'
    i18n_header = _("Audience name")


@adapter_config(name='trash',
                required=(ICinemaAudienceContainerTarget, IAdminLayer, ICinemaAudiencesTable),
                provides=IColumn)
class CinemaAudienceTrashColumn(TrashColumn):
    """Cinema audiences table trash column"""

    object_data = {
        'ams-modules': 'container',
        'ams-delete-target': 'delete-cinema-audience.json'
    }


@view_config(name='delete-cinema-audience.json',
             context=ICinemaAudienceContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=MANAGE_THEATER_PERMISSION)
def delete_cinema_audience(request):
    """Delete cinema audience"""
    return delete_container_element(request, container_factory=ICinemaAudienceContainer)


@pagelet_config(name='cinema-audiences.html',
                context=ICinemaAudienceContainerTarget, layer=IPyAMSLayer,
                permission=VIEW_THEATER_PERMISSION)
class CinemaAudiencesView(TableAdminView):
    """Cinema audiences view"""

    title = _("Cinema audiences")

    table_class = ICinemaAudiencesTable
    table_label = _("Cinema audiences list")


#
# Cinema audiences components
#

@viewlet_config(name='add-cinema-audience.action',
                context=ICinemaAudienceContainerTarget, layer=IAdminLayer, view=ICinemaAudiencesTable,
                manager=IToolbarViewletManager, weight=20,
                permission=MANAGE_THEATER_PERMISSION)
class CinemaAudienceAddAction(ContextAddAction):
    """Cinema audience add action"""

    label = _("Add audience")
    href = 'add-cinema-audience.html'


@ajax_form_config(name='add-cinema-audience.html',
                  context=ICinemaAudienceContainerTarget, layer=IPyAMSLayer,
                  permission=MANAGE_THEATER_PERMISSION)
class CinemaAudienceAddForm(AdminModalAddForm):
    """Cinema audience add form"""

    subtitle = _("New cinema audience")
    legend = _("New cinema audience properties")

    content_factory = ICinemaAudience
    fields = Fields(ICinemaAudience).omit('__parent__', '__name__', 'active')

    def add(self, obj):
        ICinemaAudienceContainer(self.context).append(obj)


@adapter_config(required=(ICinemaAudienceContainerTarget, IAdminLayer, CinemaAudienceAddForm),
                provides=IFormTitle)
def cinema_audience_add_form_title(context, request, form):
    """Cinema audience add form title"""
    return TITLE_SPAN.format(
        get_object_label(context, request, form))


@adapter_config(required=(ICinemaAudienceContainerTarget, IAdminLayer, CinemaAudienceAddForm),
                provides=IAJAXFormRenderer)
class CinemaAudienceAddFormRenderer(SimpleAddFormRenderer):
    """Cinema audience add form renderer"""

    table_factory = ICinemaAudiencesTable


@adapter_config(required=(ICinemaAudience, IAdminLayer, Interface),
                provides=IObjectLabel)
def cinema_audience_label(context, request, view):
    """Cinema audience label"""
    return context.name


@adapter_config(required=(ICinemaAudience, IAdminLayer, Interface),
                provides=IObjectHint)
def cinema_audience_hint(context, request, view):
    """Cinema audience hint"""
    return request.localizer.translate(_("Cinema audience"))


@adapter_config(required=(ICinemaAudience, IAdminLayer, ICinemaAudiencesTable),
                provides=ITableElementEditor)
class CinemaAudienceEditor(TableElementEditor):
    """Cinema audience editor"""


@ajax_form_config(name='properties.html',
                  context=ICinemaAudience, layer=IPyAMSLayer,
                  permission=VIEW_THEATER_PERMISSION)
class CinemaAudiencePropertiesEditForm(AdminModalEditForm):
    """Cinema audience properties edit form"""

    legend = _("Cinema audience properties")

    fields = Fields(ICinemaAudience).omit('__parent__', '__name__', 'active')


@adapter_config(required=(ICinemaAudience, IAdminLayer, IModalEditForm),
                provides=IFormTitle)
def cinema_audience_edit_form_title(context, request, form):
    """Cinema audience edit form title"""
    translate = request.localizer.translate
    theater = get_parent(context, IMovieTheater)
    return TITLE_SPAN_BREAK.format(
        get_object_label(theater, request, form),
        translate(_("Cinema audience: {}")).format(get_object_label(context, request, form)))


@adapter_config(required=(ICinemaAudience, IAdminLayer, CinemaAudiencePropertiesEditForm),
                provides=IAJAXFormRenderer)
class CinemaAudiencePropertiesEditFormRenderer(SimpleEditFormRenderer):
    """Cinema audience properties edit form renderer"""

    parent_interface = ICinemaAudienceContainerTarget
    table_factory = ICinemaAudiencesTable



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_msc.feature.profile.zmi module

This module defines components which are used to manage users profiles.
"""

from pyramid.events import subscriber
from pyramid.httpexceptions import HTTPNotFound
from zope.annotation.interfaces import IAnnotations
from zope.interface import Interface, Invalid, alsoProvides

from pyams_app_msc.feature.profile import IOperatorProfile, IUserProfile, USER_PROFILE_KEY
from pyams_app_msc.interfaces import MANAGE_BOOKING_PERMISSION
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces import DISPLAY_MODE, HIDDEN_MODE
from pyams_form.interfaces.form import IDataExtractedEvent, IFormContent, IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_mail.interfaces import IPrincipalMailInfo
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import IPrincipalInfo
from pyams_security.interfaces.plugin import ILocalUser, LOCKED_ACCOUNT_PASSWORD
from pyams_security.utility import get_principal
from pyams_security_views.interfaces.login import ILoginConfiguration
from pyams_skin.interfaces.viewlet import IFormHeaderViewletManager
from pyams_skin.schema.button import SubmitButton
from pyams_skin.viewlet.help import AlertMessage
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import create_object
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.registry import get_utility
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.form import IModalDisplayFormButtons, IModalEditFormButtons
from pyams_zmi.zmi.profile import UserProfileEditForm as ZMIProfileEditForm

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@ajax_form_config(name='add-user-profile.html',
                  layer=IPyAMSLayer,
                  permission=MANAGE_BOOKING_PERMISSION)
class UserProfileAddForm(AdminModalAddForm):
    """User profile add form"""

    title = _("User add form")
    legend = _("User account settings")
    autocomplete = 'off'

    fields = Fields(IUserProfile).omit('active', 'principal_id')
    content_factory = IUserProfile

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        email = self.widgets.get('email')
        if email is not None:
            email.object_data = {
                'input-mask': {
                    'mask': '*{1,64}[.*{1,64}][.*{1,64}][.*{1,63}]@*{1,63}.*{1,63}[.*{1,63}][.*{1,63}]',
                    'clearIncomplete': True
                }
            }
            alsoProvides(email, IObjectData)
        phone_number = self.widgets.get('phone_number')
        if phone_number is not None:
            phone_number.object_data = {
                'input-mask': '[+9{3}] [9]9 99 99 99 99'
            }
            alsoProvides(phone_number, IObjectData)
        structure_type = self.widgets.get('structure_type')
        if structure_type is not None:
            structure_type.prompt = True
            structure_type.prompt_message = _("Select structure type...")

    def add(self, obj):
        login_config = ILoginConfiguration(self.request.root)
        sm = get_utility(ISecurityManager)
        users_folder = sm.get(login_config.users_folder)
        if users_folder is not None:
            local_user = create_object(ILocalUser)
            local_user.self_registered = False
            local_user.login = obj.email
            local_user.email = obj.email
            local_user.firstname = obj.firstname
            local_user.lastname = obj.lastname
            local_user.company_name = obj.establishment
            local_user.password = LOCKED_ACCOUNT_PASSWORD
            local_user.generate_secret(notify=False, request=self.request)
            users_folder[local_user.login] = local_user
            principal = IPrincipalInfo(local_user)
            annotations = IAnnotations(principal)
            annotations[USER_PROFILE_KEY] = obj


@subscriber(IDataExtractedEvent, form_selector=UserProfileAddForm)
def extract_user_profile_add_form_data(event: IDataExtractedEvent):
    """Extract user profile add form data"""
    data = event.data
    email = data.get('email')
    if email:
        login_config = ILoginConfiguration(event.form.request.root)
        sm = get_utility(ISecurityManager)
        if login_config.open_registration:
            users_folder = sm.get(login_config.users_folder)
            if users_folder is not None:
                user = users_folder.get(email)
                if user is not None:
                    event.form.widgets.errors += (Invalid(_("This email address is already "
                                                            "registered!")),)


@viewlet_config(name='form-help',
                context=Interface, layer=IPyAMSLayer, view=UserProfileAddForm,
                manager=IFormHeaderViewletManager, weight=10)
class UserProfileAddFormHelp(AlertMessage):
    """User profile add form help"""

    status = 'info'

    _message = _("This form allows you to register a new user account.\n"
                 "If this user want to login, he will have to use the \"Forgotten "
                 "password\" feature to get a new activation link to set his password.")


#
# User profile edit form
#

class IUserProfileEditFormButtons(IModalEditFormButtons):
    """User profile edit form buttons interface"""

    disable = SubmitButton(name='disable',
                           title=_("Disable profile"),
                           condition=lambda form: IUserProfile(form.form_content).active)

    enable = SubmitButton(name='enable',
                          title=_("Enable profile"),
                          condition=lambda form: not IUserProfile(form.form_content).active)


@ajax_form_config(name='edit-user-profile.html',
                  layer=IPyAMSLayer,
                  permission=MANAGE_BOOKING_PERMISSION)
class UserProfileEditForm(AdminModalEditForm):
    """User profile edit form"""

    title = _("User edit form")
    legend = _("User account settings")

    fields = Fields(IUserProfile).omit('active')

    @property
    def buttons(self):
        if self.mode == DISPLAY_MODE:
            return Buttons(IModalDisplayFormButtons)
        return Buttons(IUserProfileEditFormButtons).select('disable', 'enable', 'apply', 'close')

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        request = self.request
        principal_id = (request.params.get('principal_id') or
                        request.params.get(f'{self.prefix}widgets.principal_id'))
        principal_widget = self.widgets.get('principal_id')
        if principal_widget is not None:
            principal_widget.mode = HIDDEN_MODE
            principal_widget.value = principal_id
        phone_number = self.widgets.get('phone_number')
        if phone_number is not None:
            phone_number.object_data = {
                'input-mask': '[+9{3}] [9]9 99 99 99 99'
            }
            alsoProvides(phone_number, IObjectData)
        email = self.widgets.get('email')
        if email is not None:
            email.readonly = 'readonly'
            if not email.value:
                sm = get_utility(ISecurityManager)
                principal_info = sm.get_principal(principal_id, info=False)
                if principal_info is not None:
                    mail_info = IPrincipalMailInfo(principal_info, None)
                    if mail_info is not None:
                        _title, principal_email = next(mail_info.get_addresses())
                        email.value = principal_email

    def update_actions(self):
        super().update_actions()
        translate = self.request.localizer.translate
        disable = self.actions.get('disable')
        if disable is not None:
            disable.add_class('btn-danger mr-auto')
            disable.hint = translate(_("Disabled profiles can't be used for website login anymore but can "
                                       "still be assigned new bookings"))
        enable = self.actions.get('enable')
        if enable is not None:
            enable.add_class('btn-danger mr-auto')

    @handler(IUserProfileEditFormButtons['disable'])
    def handle_disable(self, action):
        """Handle disable button"""
        principal_info = self.form_content
        IUserProfile(principal_info).active = False
        sm = get_utility(ISecurityManager)
        user = sm.get_raw_principal(principal_info.id)
        if ILocalUser.providedBy(user):
            user.activated = False
        self.finished_state.update({
            'action': action,
            'changes': user
        })

    @handler(IUserProfileEditFormButtons['enable'])
    def handle_enable(self, action):
        """Handle enable action"""
        principal_info = self.form_content
        IUserProfile(principal_info).active = True
        sm = get_utility(ISecurityManager)
        user = sm.get_raw_principal(principal_info.id)
        if ILocalUser.providedBy(user):
            user.wait_confirmation = False
            user.activated = True
        self.finished_state.update({
            'action': action,
            'changes': user
        })

    @handler(IUserProfileEditFormButtons['apply'])
    def handle_apply(self, action):
        super().handle_apply(self, action)

    def apply_changes(self, data):
        changes = super().apply_changes(data)
        if changes:
            data = data.get(self, data)
            login_config = ILoginConfiguration(self.request.root)
            sm = get_utility(ISecurityManager)
            users_folder = sm.get(login_config.users_folder)
            local_user = users_folder[data.get('email')]
            if local_user is not None:
                local_user.firstname = data.get('firstname')
                local_user.lastname = data.get('lastname')
                local_user.company_name = data.get('establishment')
        return changes


@adapter_config(required=(Interface, IAdminLayer, UserProfileEditForm),
                provides=IFormContent)
def user_profile_edit_form_content(context, request, view):
    """User profile edit form content getter"""
    principal = None
    principal_id = request.params.get('principal_id') or request.params.get(f'{view.prefix}widgets.principal_id')
    if principal_id:
        principal = get_principal(request, principal_id)
    else:
        email = request.params.get(f'{view.prefix}widgets.email')
        if email:
            login_config = ILoginConfiguration(request.root)
            sm = get_utility(ISecurityManager)
            users_folder = sm.get(login_config.users_folder)
            if users_folder is not None:
                try:
                    principal = next(users_folder.find_principals(email, exact_match=True))
                except StopIteration:
                    principal = None
    if principal is None:
        raise HTTPNotFound()
    return principal


#
# Operator profile edit form
#

@adapter_config(name='operator-profile',
                required=(Interface, IAdminLayer, ZMIProfileEditForm),
                provides=IGroup)
class OperatorProfileGroup(Group):
    """Operator profile group"""

    legend = _("Operator profile")

    fields = Fields(IOperatorProfile)


@adapter_config(required=(Interface, IAdminLayer, OperatorProfileGroup),
                provides=IFormContent)
def operator_profile_group_content(context, request, group):
    """Operator profile edit group content getter"""
    return IOperatorProfile(request.principal)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

import sys
import traceback
from datetime import datetime, timedelta

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Le

from pyams_app_msc.feature.booking import IBookingInfo
from pyams_app_msc.feature.booking.interfaces import IBookingArchiverTask
from pyams_app_msc.feature.planning.interfaces import ISession
from pyams_app_msc.shared.theater import IMovieTheater, IMovieTheaterSettings
from pyams_catalog.query import CatalogResultSet
from pyams_layer.skin import apply_skin
from pyams_scheduler.interfaces.task import TASK_STATUS_FAIL, TASK_STATUS_OK
from pyams_scheduler.task import Task
from pyams_site.interfaces import ISiteRoot
from pyams_utils.factory import factory_config, get_interface_base_name
from pyams_utils.registry import get_local_registry, get_utility, set_local_registry
from pyams_utils.request import check_request
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_zmi.skin import AdminSkin
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@factory_config(IBookingArchiverTask)
class BookingArchiverTask(Task):
    """Bookings archiver task"""

    label = _("Bookings archiver task")
    icon_class = 'fas fa-lock'

    def run(self, report, **kwargs):
        """Run sessions archiver task"""
        try:
            report.write('Sessions archiver task\n'
                         '======================\n')
            old_registry = get_local_registry()
            try:
                request = check_request()
                apply_skin(request, AdminSkin)
                root = get_parent(self, ISiteRoot)
                set_local_registry(root)
                catalog = get_utility(ICatalog)
                now = tztime(datetime.now())
                query = And(Eq(catalog['object_types'], get_interface_base_name(IBookingInfo)),
                            Le(catalog['planning_end_date'], now))
                for booking in CatalogResultSet(CatalogQuery(catalog).query(query,
                                                                            sort_index='planning_start_date')):
                    if booking.archived:
                        continue
                    theater = IMovieTheater(booking, None)
                    if theater is None:
                        continue
                    settings = IMovieTheaterSettings(theater)
                    session = ISession(booking, None)
                    if session.start_date < (now - timedelta(hours=settings.booking_retention_duration)):
                        report.write(f" - {get_object_label(session, request, self, name='text')}"
                                     f" - {get_object_label(booking, request, self)}\n")
                        booking.archived = True
            finally:
                set_local_registry(old_registry)
            return TASK_STATUS_OK, None
        except Exception:
            etype, value, tb = sys.exc_info()  # pylint: disable=invalid-name
            report.write('\n\n'
                         'An error occurred\n'
                         '=================\n')
            report.write(''.join(traceback.format_exception(etype, value, tb)))
            return TASK_STATUS_FAIL, None



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface
from zope.schema import TextLine


__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class IAdminInfo(Interface):
    """Theater administrative information interface"""

    siret_code = TextLine(title=_("SIRET code"),
                          max_length=14,
                          required=False)

    ape_code = TextLine(title=_("APE code"),
                        max_length=5,
                        required=False)

    vat_number = TextLine(title=_("Intra-community VAT number"),
                          max_length=13,
                          required=False)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.container.btree import BTreeContainer
from zope.location.interfaces import ISublocations
from zope.traversing.interfaces import ITraversable

from pyams_app_msc.shared.catalog.interfaces import CATALOG_MANAGER_KEY, ICatalogManager, ICatalogManagerTarget
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_content.shared.common.manager import BaseSharedTool
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config

__docformat__ = 'restructuredtext'

from pyams_utils.traversing import get_parent


@factory_config(ICatalogManager)
class CatalogManager(BTreeContainer, BaseSharedTool):
    """Catalog manager"""

    shared_content_menu = False

    @property
    def shared_content_workflow(self):
        parent = get_parent(self, IMovieTheater)
        return parent.shared_content_workflow


@adapter_config(required=ICatalogManagerTarget,
                provides=ICatalogManager)
def catalog_manager(context):
    """Catalog manager factory"""
    return get_annotation_adapter(context, CATALOG_MANAGER_KEY, ICatalogManager,
                                  name='++catalog++')


@adapter_config(name='catalog',
                required=ICatalogManagerTarget,
                provides=ITraversable)
class CatalogManagerTraverser(ContextAdapter):
    """Catalog manager traverser"""

    def traverse(self, name, furtherPath=None):
        """Catalog getter"""
        return ICatalogManager(self.context)


@adapter_config(name='catalog',
                required=ICatalogManagerTarget,
                provides=ISublocations)
class CatalogManagerSublocations(ContextAdapter):
    """Catalog manager sub-locations"""

    def sublocations(self):
        """Catalog manager sub-locations iterator"""
        yield from ICatalogManager(self.context).values()
